package de.duehl.math.functionviewer.ui;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Stroke;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import de.duehl.math.functionviewer.logic.Function;
import de.duehl.swing.ui.PaintableAreaCalculator;
import de.duehl.basics.logic.ErrorHandler;
import de.duehl.basics.text.NumberString;
import de.duehl.concurrent.annotations.GuardedBy;
import de.duehl.concurrent.annotations.GuardedByItSelf;

/**
 * Diese Klasse stellt die Panel dar, auf dem die Funktionen gezeichnet werden.
 *
 * @version 1.01     2019-07-03
 * @author Christian Dühl
 */

public class FunctionViewPanel extends JPanel implements RangeSelectionReactor, MouseMotionReactor {

    private static final long serialVersionUID = 196804883629301089L;

    /** Hintergrundfarbe des Zeichenbereiches. */
    private static final Color BACKGROUND_COLOR = Color.WHITE;

    /** Farbe der Koordinatenachsen. */
    private static final Color AXIS_COLOR = Color.BLACK;

    /** Breite der Koordinatenachsen. */
    private static final int AXIS_WIDTH = 1;

    /** Angestrebte Anzahl an Pixeln zwischen zwei Querstrichen auf den Koordinatenachsen. */
    private static final int PIXEL_PER_TICK = 50;

    /** Breite der Querstriche der Koordinatenachsen. */
    private static final int TICK_WIDTH = 5;

    /** Objekt zum Anzeigen von Fehlern und Warnungen. */
    private final ErrorHandler error;

    /** Liste mit den Funktionen. */
    @GuardedByItSelf
    private final List<Function> functions;

    /** Minimaler angezeigter x-Wert auf der x-Achse. */
    @GuardedBy("functions")
    private double xFrom;

    /** Minimaler angezeigter y-Wert auf der y-Achse. */
    @GuardedBy("functions")
    private double yFrom;

    /** Maximaler angezeigter x-Wert auf der x-Achse. */
    @GuardedBy("functions")
    private double xTo;

    /** Maximaler angezeigter y-Wert auf der y-Achse. */
    @GuardedBy("functions")
    private double yTo;

    /* Variablen zur Auswahl des neuen Wertebereichs per Maus: */
    private boolean rangeSelectionRunning;
    private int rangeSelectionStartPointX;
    private int rangeSelectionStartPointY;
    private int rangeSelectionDragedToX;
    private int rangeSelectionDragedToY;
    private final RangeOfValuesChangeListener rangeOfValuesChangeListener;

    /**
     * Konstruktor.
     *
     * @param error
     *            Objekt zum Anzeigen von Fehlern und Warnungen.
     * @param rangeOfValuesChangeListener
     *            Objekt, das über eine Änderung des Wertebereiches informiert
     *            wird.
     */
    public FunctionViewPanel(ErrorHandler error,
            RangeOfValuesChangeListener rangeOfValuesChangeListener) {
        this.error = error;
        this.rangeOfValuesChangeListener = rangeOfValuesChangeListener;
        functions = new ArrayList<>();
        xFrom = -5.0;
        xTo = 5.0;
        yFrom = -5.0;
        yTo = 5.0;
        rangeSelectionRunning = false;
    }

    /** Auf Panel zeichnen. Fehlerbehandlung. */
    @Override
    public void paintComponent(Graphics graphics) {
        super.paintComponent(graphics);

        try {
            paintComponentsInternal(graphics);
        }
        catch (Exception exception) {
            error.error("Es ist ein Fehler beim Zeichnen aufgetreten!",
                    exception);
        }
    }

    /** Auf Panel zeichnen. */
    private void paintComponentsInternal(Graphics graphics) {
        /* Die Maße der wirklichen Zeichenfläche ermitteln: */
        PaintableAreaCalculator paint = new PaintableAreaCalculator(this);

        /* Hintergrund einfärben: */
        paintComponentsBackground(graphics, paint);

        /* Zu zeichnenden Wertebereich ermitteln: */
        double xMin;
        double xMax;
        double yMin;
        double yMax;
        synchronized(functions) {
            xMin = xFrom;
            xMax = xTo;
            yMin = yFrom;
            yMax = yTo;
        }

        /* Koordinatenachsen einzeichnen: */
        paintAxesOfCoordinates(graphics, paint, xMin, xMax, yMin, yMax);

        /* Funktionen einzeichnen: */
        List<Function> copiedFunctions = getCopyOfFunctions();
        for (Function function : copiedFunctions) {
            paintComponentsFunction(graphics, paint, function, xMin, xMax, yMin, yMax);
        }

        /* Auswahl des neuen Wertebereiches einzeichnen: */
        if (rangeSelectionRunning) {
            int rangePixelXMin = rangeSelectionStartPointX;
            int rangePixelYMin = rangeSelectionStartPointY;
            int rangePixelXMax = rangeSelectionDragedToX;
            int rangePixelYMax = rangeSelectionDragedToY;

            /* vertauschen, falls nötig: */
            //if (rangePixelXMin > rangePixelXMax) {
            //
            //}

            drawRect(graphics, rangePixelXMin, rangePixelYMin, rangePixelXMax, rangePixelYMax,
                    Color.RED, LineStyle.DOTTED, 1);
        }
    }

    /**
     * Zeichnet den Hintergrund ein.
     *
     * @param graphics
     *            Der <code>Graphics</code> Kontext in den gezeichnet wird.
     * @param paint
     *            Die Maße der wirklichen Zeichenfläche.
     */
    private void paintComponentsBackground(Graphics graphics,
            PaintableAreaCalculator paint) {
        //setBackground(BACKGROUND_COLOR);
        // Reicht nicht aus, weil es den Rahmen übermalt...

        graphics.setColor(BACKGROUND_COLOR);
        graphics.fillRect(paint.getUpperLeftX(), paint.getUpperLeftY(),
                paint.getPaintableWidth(), paint.getPaintableHeight());
    }

    /**
     * Zeichnet die Koordinatenachsen ein.
     *
     * @param graphics
     *            Der <code>Graphics</code> Kontext in den gezeichnet wird.
     * @param paint
     *            Die Maße der wirklichen Zeichenfläche.
     * @param xMin
     *            Minimaler angezeigter x-Wert auf der x-Achse (als Zahl).
     * @param xMax
     *            Maximaler angezeigter x-Wert auf der x-Achse (als Zahl).
     * @param yMin
     *            Minimaler angezeigter y-Wert auf der y-Achse (als Zahl).
     * @param yMax
     *            Maximaler angezeigter y-Wert auf der y-Achse (als Zahl).
     */
    private void paintAxesOfCoordinates(Graphics graphics, PaintableAreaCalculator paint,
            double xMin, double xMax, double yMin, double yMax) {
        /* Ausmaße und Koordinaten der wirklichen Zeichenfläche ermitteln: */
        int upperLeftX = paint.getUpperLeftX();
        int upperLeftY = paint.getUpperLeftY();
        int lowerRightX = paint.getLowerRightX();
        int lowerRightY = paint.getLowerRightY();

        paintXAxis(graphics, xMin, xMax, yMin, yMax, upperLeftX, upperLeftY, lowerRightX,
                lowerRightY);
        paintYAxis(graphics, xMin, xMax, yMin, yMax, upperLeftX, upperLeftY, lowerRightX,
                lowerRightY);
    }

    /**
     * Zeichnet die x-Achse ein. ist die Achse eigentlich nicht im Bild, so wird sie an dem Rand,
     * in dessen Richtung sie liegt, gestrichelt eingezeichnet.
     *
     * @param graphics
     *            Der <code>Graphics</code> Kontext in den gezeichnet wird.
     * @param xMin
     *            Minimaler angezeigter x-Wert auf der x-Achse (als Zahl).
     * @param xMax
     *            Maximaler angezeigter x-Wert auf der x-Achse (als Zahl).
     * @param yMin
     *            Minimaler angezeigter y-Wert auf der y-Achse (als Zahl).
     * @param yMax
     *            Maximaler angezeigter y-Wert auf der y-Achse (als Zahl).
     * @param upperLeftX
     *            x-Koordinate der linken oberen Ecke des darstellbaren Bereichs in Pixeln.
     * @param upperLeftY
     *            y-Koordinate der linken oberen Ecke des darstellbaren Bereichs in Pixeln.
     * @param lowerRightX
     *            x-Koordinate der rechten unteren Ecke des darstellbaren Bereichs in Pixeln.
     * @param lowerRightY
     *            y-Koordinate der rechten unteren Ecke des darstellbaren Bereichs in Pixeln.
     */
    private void paintXAxis(Graphics graphics, double xMin, double xMax, double yMin, double yMax,
            int upperLeftX, int upperLeftY, int lowerRightX, int lowerRightY) {

        /*
         * Zunächst wird ermittelt, ob die x-Koordinatenachse oben oder unten am Rand oder ganz
         * normal im Inneren des darstellbaren Bereiches eingezeichnet wird:
         */
        boolean showTop;
        boolean showBottom;
        if (yMax < 0d) {
            showTop = true;
            showBottom = false;
        }
        else if (yMin > 0d) {
            showTop = false;
            showBottom = true;
        }
        else {
            showTop = false;
            showBottom = false;
        }

        /* Dann wird die y-Koordinate berechnet, an der die x-Achse eingezeichnet wird: */
        int yCoordinate;
        if (showBottom) {
            yCoordinate = lowerRightY;
        }
        else if (showTop) {
            yCoordinate = upperLeftY;
        }
        else {
            yCoordinate = calcuateYCoordinate(0, yMin, yMax, upperLeftY, lowerRightY);
        }

        /* Nun wird die Achse eingezeichnet: */
        LineStyle style = showTop || showBottom ? LineStyle.DASHED : LineStyle.NORMAL;
        drawLine(graphics, upperLeftX, yCoordinate, lowerRightX, yCoordinate, AXIS_COLOR, style,
                AXIS_WIDTH);

        /* Beschriftung der x-Achse erzeugen. */
        {
            int upperAndLower = TICK_WIDTH - AXIS_WIDTH / 2;
            int tickFromY = yCoordinate - upperAndLower;
            int tickToY = yCoordinate + upperAndLower;
            if (tickFromY < upperLeftY) {
                int move = upperLeftX - tickFromY;
                tickFromY += move;
                tickToY += move;
            }
            else if (tickToY > lowerRightY) {
                int move = tickToY - lowerRightY;
                tickFromY -= move;
                tickToY -= move;
            }

            double tickDistance = calculateTickDistance(
                    lowerRightX - upperLeftX,
                    xMax - xMin);

            double start = Math.floor(xMin);
            double end = Math.ceil(xMax);
            int tooNear = 10; // zu Nah am Rand in Pixeln
            for (double x = start; x <= end; x += tickDistance) {
                //System.out.println("x = " + x);
                if (x >= xMin && x <= xMax) {
                    int xCoordinate = calcuateXCoordinate(x, xMin, xMax,
                            upperLeftX, lowerRightX);
                    if (Math.abs(xCoordinate - upperLeftX) < tooNear
                            || Math.abs(xCoordinate - lowerRightX) < tooNear) {
                        continue;
                    }
                    drawLine(graphics, xCoordinate, tickFromY, xCoordinate,
                            tickToY, AXIS_COLOR, style, AXIS_WIDTH);
                    if (!near(x, 0.0d)) {
                        String number = NumberString.removeZeroEnding(Double.toString(x));
                        Font font = graphics.getFont();
                        int numberY;
                        if (tickToY == lowerRightY) {
                            numberY = tickFromY - font.getSize();
                        }
                        else {
                            numberY = tickToY + 17;
                        }
                        FontMetrics fontMetrics = graphics.getFontMetrics(font);
                        int numberWidth = SwingUtilities.computeStringWidth(
                                fontMetrics, number);
                        int numberX = xCoordinate - numberWidth/2;
                        graphics.drawString(number, numberX, numberY);
                    }
                }
            }

            /* Pfeilspitze und Beschriftung einzeichnen: */
            if (!showTop && !showBottom) {
                /* Spitze: */
                int p1X = lowerRightX;           // Rechter Punkt
                int p1Y = yCoordinate;           // des Dreiecks

                int p2X = p1X - TICK_WIDTH;      // oberer, linker Punkt
                int p2Y = p1Y - upperAndLower;   // des Dreiecks

                int p3X = p1X - TICK_WIDTH;      // unterer, linker Punkt
                int p3Y = p1Y + upperAndLower;   // des Dreiecks

                Polygon polygon = new Polygon();
                polygon.addPoint(p1X, p1Y);
                polygon.addPoint(p2X, p2Y);
                polygon.addPoint(p3X, p3Y);
                graphics.fillPolygon(polygon);

                /* Beschriftung: */
                String axisLabel = "x";
                Font font = graphics.getFont();
                FontMetrics fontMetrics = graphics.getFontMetrics(font);
                int numberWidth = SwingUtilities.computeStringWidth(
                        fontMetrics, axisLabel);
                int axisLabelX = p3X - numberWidth/2 - 2;
                int axisLabelY = yCoordinate + upperAndLower + 15;
                graphics.drawString(axisLabel, axisLabelX, axisLabelY);
            }
        }
    }

    /**
     * Zeichnet die y-Achse ein. ist die Achse eigentlich nicht im Bild, so wird sie an dem Rand,
     * in dessen Richtung sie liegt, gestrichelt eingezeichnet.
     *
     * @param graphics
     *            Der <code>Graphics</code> Kontext in den gezeichnet wird.
     * @param xMin
     *            Minimaler angezeigter x-Wert auf der x-Achse (als Zahl).
     * @param xMax
     *            Maximaler angezeigter x-Wert auf der x-Achse (als Zahl).
     * @param yMin
     *            Minimaler angezeigter y-Wert auf der y-Achse (als Zahl).
     * @param yMax
     *            Maximaler angezeigter y-Wert auf der y-Achse (als Zahl).
     * @param upperLeftX
     *            x-Koordinate der linken oberen Ecke des darstellbaren Bereichs in Pixeln.
     * @param upperLeftY
     *            y-Koordinate der linken oberen Ecke des darstellbaren Bereichs in Pixeln.
     * @param lowerRightX
     *            x-Koordinate der rechten unteren Ecke des darstellbaren Bereichs in Pixeln.
     * @param lowerRightY
     *            y-Koordinate der rechten unteren Ecke des darstellbaren Bereichs in Pixeln.
     */
    private void paintYAxis(Graphics graphics, double xMin, double xMax, double yMin, double yMax,
            int upperLeftX, int upperLeftY, int lowerRightX, int lowerRightY) {
        /*
         * Zunächst wird ermittelt, ob die y-Koordinatenachse rechts oder links am Rand oder ganz
         * normal im Inneren des darstellbaren Bereiches eingezeichnet wird:
         */
        boolean showRight;
        boolean showLeft;
        if (xMax < 0d) {
            showRight = true;
            showLeft = false;
        }
        else if (xMin > 0d) {
            showRight = false;
            showLeft = true;
        }
        else {
            showRight = false;
            showLeft = false;
        }

        /* Dann wird die x-Koordinate berechnet, an der die y-Achse eingezeichnet wird: */
        int xCoordinate;
        if (showLeft) {
            xCoordinate = upperLeftX;
        }
        else if (showRight) {
            xCoordinate = lowerRightX;
        }
        else {
            xCoordinate = calcuateXCoordinate(0, xMin, xMax,
                    upperLeftX, lowerRightX);
        }

        /* Nun wird die Achse eingezeichnet: */
        LineStyle style = showRight || showLeft ? LineStyle.DASHED : LineStyle.NORMAL;
        drawLine(graphics, xCoordinate, upperLeftY, xCoordinate, lowerRightY,
                AXIS_COLOR, style, AXIS_WIDTH);

        /* Beschriftung der y-Achse erzeugen. */
        {
            int leftAndRight = TICK_WIDTH - AXIS_WIDTH / 2;
            int tickFromX = xCoordinate - leftAndRight;
            int tickToX = xCoordinate + leftAndRight;
            if (tickFromX < upperLeftX) {
                int move = upperLeftX - tickFromX;
                tickFromX += move;
                tickToX += move;
            }
            else if (tickToX > lowerRightX) {
                int move = tickToX - lowerRightX;
                tickFromX -= move;
                tickToX -= move;
            }

            double tickDistance = calculateTickDistance(
                    lowerRightY - upperLeftY,
                    yMax - yMin);

            double start = Math.floor(yMin);
            double end = Math.ceil(yMax);
            int tooNear = 10; // zu Nah am Rand in Pixeln
            for (double y = start; y <= end; y += tickDistance) {
                //System.out.println("y = " + y);
                if (y >= yMin && y <= yMax) {
                    int yCoordinate = calcuateYCoordinate(y, yMin, yMax,
                            upperLeftY, lowerRightY);
                    if (Math.abs(yCoordinate - upperLeftY) < tooNear
                            || Math.abs(yCoordinate - lowerRightY) < tooNear) {
                        continue;
                    }
                    drawLine(graphics, tickFromX, yCoordinate, tickToX,
                            yCoordinate, AXIS_COLOR, style, AXIS_WIDTH);
                    if (!near(y, 0.0d)) {
                        String number = NumberString.removeZeroEnding(Double.toString(y));
                        int numberX;
                        if (tickFromX == upperLeftX) {
                            numberX = tickToX + 3;
                        }
                        else {
                            Font font = graphics.getFont();
                            FontMetrics fontMetrics = graphics.getFontMetrics(
                                    font);
                            int numberWidth = SwingUtilities.computeStringWidth(
                                    fontMetrics, number);
                            numberX = tickFromX - 5 - numberWidth;
                        }
                        int numberY = yCoordinate + 5;
                        graphics.drawString(number, numberX, numberY);
                    }
                }
            }

            /* Pfeilspitze und Beschriftung einzeichnen: */
            if (!showRight && !showLeft) {
                /* Spitze: */
                int p1X = xCoordinate;        // Oberer Punkt des Dreiecks
                int p1Y = upperLeftY;

                int p2X = p1X - leftAndRight; // linker, unterer Punkt
                int p2Y = p1Y + TICK_WIDTH;   // des Dreiecks
                // + wegen y-Verdrehung

                int p3X = p1X + leftAndRight; // rechter, unterer Punkt
                int p3Y = p1Y + TICK_WIDTH;   // des Dreiecks
                // + wegen y-Verdrehung

                Polygon polygon = new Polygon();
                polygon.addPoint(p1X, p1Y);
                polygon.addPoint(p2X, p2Y);
                polygon.addPoint(p3X, p3Y);
                graphics.fillPolygon(polygon);

                /* Beschriftung: */
                String axisLabel = "y";

                Font font = graphics.getFont();
                FontMetrics fontMetrics = graphics.getFontMetrics(font);
                int numberWidth = SwingUtilities.computeStringWidth(
                        fontMetrics, axisLabel);
                int axisLabelX = xCoordinate - leftAndRight - 5 - numberWidth;
                int axisLabelY = p3Y + TICK_WIDTH;
                graphics.drawString(axisLabel, axisLabelX, axisLabelY);
            }
        }
    }

    /**
     * Berechnet, ob zwei double-Werte nah genug aneinander sind.
     *
     * @param valueOne
     *            Erster double-Wert.
     * @param valueTwo
     *            Zweiter double-Wert.
     * @return Wahrheitswert für annähernde Gleichheit der beiden Werte.
     */
    private boolean near(double valueOne, double valueTwo) {
        double epsilon = 0.000001;
        double distance = Math.abs(valueOne - valueTwo);
        return distance < epsilon;
    }

    /**
     * Berechnet die Abstände der in die Achse einzuzeichenden Querstriche und Zahlen.
     *
     * @param distanceInPixel
     *            Abstand zwischen größtem und kleinstem Wert des darstellbaren Bereiches in
     *            Pixeln.
     * @param distanceAsNumber
     *            Abstand zwischen größtem und kleinstem Wert des darstellbaren Bereiches in
     *            Zahlen.
     * @return Abstand der einzuzeichenden Querstriche.
     */
    private double calculateTickDistance(int distanceInPixel, double distanceAsNumber) {
        /*
         * Zuerst wird die Anzahl an gewünschten Ticks (ein Tick ist später ein Querstrich mit
         * einer Zahl daran) berechnet:
         */
        int wantedTicks = distanceInPixel / PIXEL_PER_TICK;

        /* Dann wird die Anzahl der Ticks berechnet, wenn man jede ganze Zahl einzeichnen würde: */
        double tickDistance = 1.0d;
        int ticks = (int)(distanceAsNumber / tickDistance);

        /*
         * Abhängig davon, ob wir zu viele oder zu wenige Ticks haben, wird ein Factor gebildet,
         * mit dem die tickDistance dann multipliziert wird:
         */
        double tickDistanceFactor;
        if  (ticks < wantedTicks) {
            tickDistanceFactor = 0.5;
        }
        else {
            tickDistanceFactor = 2;
        }

        /*
         * Es wird nun nach der kleinsten Abweichung der Anzahl der Ticks von der Anzahl der
         * gewünschten Ticks gesucht. Solange die Abweichung pro Schritt kleiner wird, machen wir
         * weiter, wird sie größer, nehmen wir den vorigen Wert (dessen Abweichung kleiner ist):
         */
        {
            double oldDelta = Math.abs(ticks - wantedTicks);
            double oldTickDistace = tickDistance;
            int oldTicks = ticks;

            double actualDelta;
            boolean running = true;

            while (running) {
                tickDistance *= tickDistanceFactor;
                ticks = (int)(distanceAsNumber / tickDistance);
                actualDelta = Math.abs(ticks - wantedTicks);

                if (actualDelta > oldDelta) {
                    running = false;
                    tickDistance = oldTickDistace;
                    ticks = oldTicks;
                }
                else {
                    oldDelta = actualDelta;
                    oldTickDistace = tickDistance;
                    oldTicks = ticks;
                }
            }
        }

        return tickDistance;
    }

    /**
     * Zeichnet eine Funktion ein.
     *
     * @param graphics
     *            Der <code>Graphics</code> Kontext in den gezeichnet wird.
     * @param paint
     *            Die Maße der wirklichen Zeichenfläche.
     * @param function
     *            Einzuzeichnende Funktion.
     * @param xMin
     *            Minimaler angezeigter x-Wert auf der x-Achse (als Zahl).
     * @param xMax
     *            Maximaler angezeigter x-Wert auf der x-Achse (als Zahl).
     * @param yMin
     *            Minimaler angezeigter y-Wert auf der y-Achse (als Zahl).
     * @param yMax
     *            Maximaler angezeigter y-Wert auf der y-Achse (als Zahl).
     */
    private void paintComponentsFunction(Graphics graphics, PaintableAreaCalculator paint,
            Function function, double xMin, double xMax, double yMin, double yMax) {
        int xMinPixel = paint.getUpperLeftX();
        int xMaxPixel = paint.getLowerRightX();
        int yMinPixel = paint.getUpperLeftY();
        int yMaxPixel = paint.getLowerRightY();
        double numberWidth = xMax - xMin;
        double pixelWidth = xMaxPixel - xMinPixel;
        graphics.setColor(function.getColor());
        int lastX = Integer.MIN_VALUE;
        int lastY = Integer.MIN_VALUE;
        for (int xPixel = xMinPixel; xPixel <= xMaxPixel; ++xPixel) {
            /*
             * x - xFrom entspricht
             * (x - xFrom) / numberWidth entspricht
             *     (xPixel - xMinPixel) / pixelWidth.
             * Also:
             * (x - xFrom) / numberWidth = (xPixel - xMinPixel) / pixelWidth
             * x - xFrom = numberWidth * (xPixel - xMinPixel) / pixelWidth
             * x = xFrom + numberWidth * (xPixel - xMinPixel) / pixelWidth
             * x = xFrom + (xPixel - xMinPixel) * numberWidth / pixelWidth
             */
            double x = xMin + (xPixel - xMinPixel) * numberWidth / pixelWidth;
            double y = function.calculate(x);
            if (y >= yMin && y <= yMax) {
                int yPixel = calcuateYCoordinate(y, yMin, yMax, yMinPixel, yMaxPixel);
                //graphics.drawRect(xPixel, yPixel, 1, 1);
                if (lastX == Integer.MIN_VALUE) {
                    lastX = xPixel;
                    lastY = yPixel;
                }
                drawLine(graphics, lastX, lastY, xPixel, yPixel, function.getColor(),
                        LineStyle.NORMAL, 1);
                lastX = xPixel;
                lastY = yPixel;
            }
            else {
                lastX = Integer.MIN_VALUE;
                lastY = Integer.MIN_VALUE;
            }
        }
    }

    /**
     * Zeichnet eine Linie auf das Panel.
     *
     * @param graphics
     *            Der <code>Graphics</code> Kontext in den gezeichnet wird.
     * @param xFrom
     *            x-Wert des Startpunkts (in Pixeln).
     * @param yFrom
     *            y-Wert des Startpunkts (in Pixeln).
     * @param xTo
     *            x-Wert des Endpunkts (in Pixeln).
     * @param yTo
     *            y-Wert des Endpunkts (in Pixeln).
     * @param color
     *            Farbe der Linie
     * @param style
     *            Stil der Linie.
     * @param width
     *            Dicke der Linie.
     */
    private void drawLine(Graphics graphics, int xFrom, int yFrom, int xTo, int yTo, Color color,
            LineStyle style, double width) {
        Stroke stroke = generateStrokeFromStyle(style, width);
        Graphics2D g2d = (Graphics2D) graphics;
        g2d.setStroke(stroke);
        g2d.setColor(color);
        g2d.drawLine(xFrom, yFrom, xTo, yTo);
    }

    /**
     * Zeichnet ein Rechteck auf das Panel.
     *
     * @param graphics
     *            Der <code>Graphics</code> Kontext in den gezeichnet wird.
     * @param xFrom
     *            x-Wert des Startpunkts (in Pixeln).
     * @param yFrom
     *            y-Wert des Startpunkts (in Pixeln).
     * @param xTo
     *            x-Wert des Endpunkts (in Pixeln).
     * @param yTo
     *            y-Wert des Endpunkts (in Pixeln).
     * @param color
     *            Farbe der Linie
     * @param style
     *            Stil der Linie.
     * @param width
     *            Dicke der Linie.
     */
    private void drawRect(Graphics graphics, int xFrom, int yFrom, int xTo, int yTo, Color color,
            LineStyle style, double width) {
        Stroke stroke = generateStrokeFromStyle(style, width);
        Graphics2D g2d = (Graphics2D) graphics;
        g2d.setStroke(stroke);
        g2d.setColor(color);
        int x1 = xFrom;
        int x2 = xTo;
        if (x2 < x1) {
            x1 = xTo;
            x2 = xFrom;
        }
        int y1 = yFrom;
        int y2 = yTo;
        if (y2 < y1) {
            y1 = yTo;
            y2 = yFrom;
        }
        g2d.drawRect(x1, y1, x2-x1, y2-y1);
    }

    /**
     * Erzeugt aus der Dicke der Linie und dem Stil ein BasicStroke.
     *
     * @param style
     *            Stil der Linie
     * @param width
     *            Dicke der Linie
     * @return BasicStroke für die Darstellung
     */
    private Stroke generateStrokeFromStyle(LineStyle style, double width) {
        Stroke stroke;
        switch (style) {
            default:
            case NORMAL:
                stroke = new BasicStroke((float) width);
                break;
            case DASHED:
                stroke = new BasicStroke(
                        (float) width,              // Width
                        BasicStroke.CAP_BUTT,       // End cap
                        BasicStroke.JOIN_MITER,     // Join style
                        10.0f,                      // Miter limit
                        new float[] {8.0f, 8.0f},   // Dash pattern
                        0.0f);                      // Dash phase
                    break;
            case DOTTED:
                stroke = new BasicStroke(
                        (float) width,              // Width
                        BasicStroke.CAP_BUTT,       // End cap
                        BasicStroke.JOIN_MITER,     // Join style
                        10.0f,                      // Miter limit
                        new float[] {4.0f, 4.0f},   // Dash pattern
                        0.0f);                      // Dash phase
                    break;
        }
        //new float[] { 21.0f, 9.0f, 3.0f, 9.0f }, // Dot-Dash pattern
        return stroke;
    }

    /**
     * Berechnet die passende x-Coordinate in Pixeln auf dem Panel.
     *
     * @param value
     *            Gesuchter x-Wert als Zahl.
     * @param xMin
     *            Minimaler x-Wert als Zahl.
     * @param xMax
     *            Maximaler x-Wert als Zahl.
     * @param xMinPixel
     *            Minimaler x-Wert als Pixel.
     * @param xMaxPixel
     *            Maximaler x-Wert als Pixel.
     * @return Den Werten entsprechende Pixelkoordinate für den gesuchten x-Wert.
     */
    private int calcuateXCoordinate(double value, double xMin, double xMax, int xMinPixel,
            int xMaxPixel) {
        if (value < xMin || value > xMax) {
            throw new IllegalArgumentException("Der x-Wert " + value
                    + " liegt nicht innerhalb von [" + xMin + ", " + xMax
                    + "].");
        }
        /* xMin entspricht xMinPixel, xMax entspricht xMaxPixel.
         *
         *            pixelValue
         * xMinPixel------|---------------------------------xMaxPixel
         *                |
         *                |
         *                |
         * xMin-----------|--------------------------------------xMax
         *              value
         *
         * numberWidth = xMax - xMin
         * pixelWidth = xMaxPixel - xMinPixel
         * numberWidth entspricht pixelWidth.
         *
         * (value - xMin) / numberWidth entspricht
         *      (pixelValue - xMinPixel) / pixelWidth.
         * Also:
         * (value - xMin) / numberWidth = (pixelValue - xMinPixel) / pixelWidth
         * pixelWidth * (value - xMin) / numberWidth = pixelValue - xMinPixel
         * xMinPixel + pixelWidth * (value - xMin) / numberWidth = pixelValue
         * pixelValue = xMinPixel + pixelWidth * (value - xMin) / numberWidth
         *
         */
        double numberWidth = xMax - xMin;
        int pixelWidth = xMaxPixel - xMinPixel;
        int pixelValue = (int) (xMinPixel + pixelWidth * (value - xMin) / numberWidth);
        return pixelValue;
    }

    /**
     * Berechnet die passende y-Coordinate in Pixeln auf dem Panel.
     *
     * @param value
     *            Gesuchter y-Wert als Zahl.
     * @param yMin
     *            Minimaler y-Wert als Zahl.
     * @param yMax
     *            Maximaler x-Wert als Zahl.
     * @param yMinPixel
     *            Minimaler y-Wert als Pixel.
     * @param yMaxPixel
     *            Maximaler y-Wert als Pixel.
     * @return Den Werten entsprechende Pixelkoordinate für den gesuchten y-Wert.
     */
    private int calcuateYCoordinate(double value, double yMin, double yMax, int yMinPixel,
            int yMaxPixel) {
        if (value < yMin || value > yMax) {
            throw new IllegalArgumentException("Der y-Wert " + value
                    + " liegt nicht innerhalb von [" + yMin + ", " + yMax
                    + "].");
        }
        /* yMin entspricht yMinPixel, yMax entspricht yMaxPixel.
         *
         *           yMinPixel              yMax
         *               |                    |
         *               |                    |
         *               |                    |
         *               |                    |
         *               |                    |
         *               |                    |
         *               |                    |
         *               |                    |
         *  pixelValue -------------------------- value
         *               |                    |
         *               |                    |
         *               |                    |
         *               |                    |
         *           yMaxPixel              yMin
         *
         * numberHeight = yMax - yMin
         * pixelHeight = xMaxPixel - xMinPixel
         * numberHeight entspricht pixelHeight.
         *
         * (value - yMin) / numberHeight entspricht
         *     (yMaxPixel - pixelValue) / pixelHeight.
         * Also:
         * (value - yMin) / numberHeight = (yMaxPixel - pixelValue) / pixelHeight
         * pixelHeight * (value - yMin) / numberHeight = yMaxPixel - pixelValue
         * pixelValue + pixelHeight * (value - yMin) / numberHeight = yMaxPixel
         * pixelValue = yMaxPixel - pixelHeight * (value - yMin) / numberHeight
         */
        double numberHeight = yMax - yMin;
        int pixelHeight = yMaxPixel - yMinPixel;
        int pixelValue = (int) (yMaxPixel - pixelHeight * (value - yMin) / numberHeight);
        return pixelValue;
    }

    /**
     * Berechnet den passenden Zahlenwert für die x-Coordinate auf dem Panel.
     *
     * @param pixelValue
     *            Gesuchter x-Wert als Pixel.
     * @param xMin
     *            Minimaler x-Wert als Zahl.
     * @param xMax
     *            Maximaler x-Wert als Zahl.
     * @param xMinPixel
     *            Minimaler x-Wert als Pixel.
     * @param xMaxPixel
     *            Maximaler x-Wert als Pixel.
     * @return Den Werten entsprechende Zahlenwert für den gesuchten x-Wert.
     */
    private double calcuateXNumber(int pixelValue, double xMin, double xMax, int xMinPixel,
            int xMaxPixel) {
        if (pixelValue < xMinPixel || pixelValue > xMaxPixel) {
            throw new IllegalArgumentException("Der x-Wert " + pixelValue
                    + " liegt nicht innerhalb von [" + xMinPixel + ", "
                    + xMaxPixel + "].");
        }
        /* xMin entspricht xMinPixel, xMax entspricht xMaxPixel.
         *
         *            pixelValue
         * xMinPixel------|---------------------------------xMaxPixel
         *                |
         *                |
         *                |
         * xMin-----------|--------------------------------------xMax
         *              value
         *
         * numberWidth = xMax - xMin
         * pixelWidth = xMaxPixel - xMinPixel
         * numberWidth entspricht pixelWidth.
         *
         * (value - xMin) / numberWidth entspricht
         *      (pixelValue - xMinPixel) / pixelWidth.
         * Also:
         * (value - xMin) / numberWidth = (pixelValue - xMinPixel) / pixelWidth
         * value - xMin = numberWidth * (pixelValue - xMinPixel) / pixelWidth
         * value = xMin + numberWidth * (pixelValue - xMinPixel) / pixelWidth
         *
         */
        double numberWidth = xMax - xMin;
        int pixelWidth = xMaxPixel - xMinPixel;
        double value = xMin + numberWidth * (pixelValue - xMinPixel) / pixelWidth;
        return value;
    }

    /**
     * Berechnet den passenden Zahlenwert für die y-Coordinate auf dem Panel.
     *
     * @param pixelValue
     *            Gesuchter y-Wert als Pixel.
     * @param yMin
     *            Minimaler y-Wert als Zahl.
     * @param yMax
     *            Maximaler x-Wert als Zahl.
     * @param yMinPixel
     *            Minimaler y-Wert als Pixel.
     * @param yMaxPixel
     *            Maximaler y-Wert als Pixel.
     * @return Den Werten entsprechende Zahlenwert für den gesuchten
     *         y-Wert.
     */
    private double calcuateYNumber(int pixelValue, double yMin, double yMax, int yMinPixel,
            int yMaxPixel) {
        if (pixelValue < yMinPixel || pixelValue > yMaxPixel) {
            throw new IllegalArgumentException("Der y-Wert " + pixelValue
                    + " liegt nicht innerhalb von [" + yMinPixel + ", "
                    + yMaxPixel + "].");
        }
        /* yMin entspricht yMinPixel, yMax entspricht yMaxPixel.
         *
         *           yMinPixel              yMax
         *               |                    |
         *               |                    |
         *               |                    |
         *               |                    |
         *               |                    |
         *               |                    |
         *               |                    |
         *               |                    |
         *  pixelValue -------------------------- value
         *               |                    |
         *               |                    |
         *               |                    |
         *               |                    |
         *           yMaxPixel              yMin
         *
         * numberHeight = yMax - yMin
         * pixelHeight = yMaxPixel - yMinPixel
         * numberHeight entspricht pixelHeight.
         *
         * (value - yMin) / numberHeight entspricht
         *     (yMaxPixel - pixelValue) / pixelHeight.
         * Also:
         * (value - yMin) / numberHeight = (yMaxPixel - pixelValue) / pixelHeight
         * value - yMin = numberHeight * (yMaxPixel - pixelValue) / pixelHeight
         * value = yMin + numberHeight * (yMaxPixel - pixelValue) / pixelHeight
         */
        double numberHeight = yMax - yMin;
        int pixelHeight = yMaxPixel - yMinPixel;
        double value = yMin + numberHeight * (yMaxPixel - pixelValue) / pixelHeight;
        return value;
    }

    public double getxFrom() {
        return xFrom;
    }

    public double getyFrom() {
        return yFrom;
    }

    public double getxTo() {
        return xTo;
    }

    public double getyTo() {
        return yTo;
    }

    public void setxFrom(double xFrom) {
        this.xFrom = xFrom;
    }

    public void setyFrom(double yFrom) {
        this.yFrom = yFrom;
    }

    public void setxTo(double xTo) {
        this.xTo = xTo;
    }

    public void setyTo(double yTo) {
        this.yTo = yTo;
    }

    /**
     * Setzt den Startpunkt für eine Auswahl des Bildschirmbereiches.
     *
     * @param x
     *            X-Koordinate des Startpunktes.
     * @param y
     *            Y-Koordinate des Startpunktes.
     */
    @Override
    public void setRangeSelectionStartPoint(int x, int y) {
        //System.out.println("setRangeSelectionStartPoint(" + x + ", " + y + ")");

        rangeSelectionRunning = true;
        rangeSelectionStartPointX = x;
        rangeSelectionStartPointY = y;
        rangeSelectionDragedToX = x;
        rangeSelectionDragedToY = y;

        paintFunctionViewPanelNew();
    }

    /**
     * Teilt mit, wohin die Maus bewegt wurde.
     *
     * @param mouseX
     *            X-Koordinate der Maus.
     * @param mouseY
     *            Y-Koordinate der Maus.
     */
    @Override
    public void mouseDragedTo(int mouseX, int mouseY) {
        rangeSelectionDragedToX = mouseX;
        rangeSelectionDragedToY = mouseY;

        paintFunctionViewPanelNew();
    }

    /**
     * Übergibt den ausgewählten Bereich in Bildschirmkoordinaten und setzt den
     * neuen Wertebereich.
     *
     * @param xPixelMin
     *            x-Koordinate der linken, oberen Ecke.
     * @param yPixelMin
     *            y-Koordinate der linken, oberen Ecke.
     * @param xPixelMax
     *            x-Koordinate der rechten, unteren Ecke.
     * @param yPixelMax
     *            y-Koordinate der rechten, unteren Ecke.
     */
    @Override
    public void setRangeSelection(int xPixelMin, int yPixelMin, int xPixelMax,
            int yPixelMax) {
System.out.println("setRangeSelection(" + xPixelMin + ", " + yPixelMin
        + ", " + xPixelMax + ", " + yPixelMax + ")");

        if (xPixelMin == xPixelMax || yPixelMin == yPixelMax) {
            paintFunctionViewPanelNew();
            return;
        }

        rangeSelectionRunning = false;

        /* Die Maße der wirklichen Zeichenfläche ermitteln: */
        PaintableAreaCalculator paint = new PaintableAreaCalculator(this);

        /* Ausmaße und Koordinaten der wirklichen Zeichenfläche ermitteln: */
        int upperLeftX = paint.getUpperLeftX();
        int upperLeftY = paint.getUpperLeftY();
        int lowerRightX = paint.getLowerRightX();
        int lowerRightY = paint.getLowerRightY();
System.out.println("upperLeftX = " + upperLeftX);
System.out.println("upperLeftY = " + upperLeftY);
System.out.println("lowerRightX = " + lowerRightX);
System.out.println("lowerRightY = " + lowerRightY);

        if (xPixelMin < upperLeftX || yPixelMin < upperLeftY
                || xPixelMax > lowerRightX || yPixelMax > lowerRightY) {
            paintFunctionViewPanelNew();
            return;
        }

        double delta = 0.00001d;
        if (xTo - xFrom < delta) {
            paintFunctionViewPanelNew();
            error.error("Die x-Werte werden zu klein.");
            return;
        }
        if (yTo - yFrom < delta) {
            paintFunctionViewPanelNew();
            error.error("Die y-Werte werden zu klein.");
            return;
        }

        /* Bisherigen zu zeichnenden Wertebereich ermitteln: */
        double xMin;
        double xMax;
        double yMin;
        double yMax;
        synchronized(functions) {
            xMin = xFrom;
            xMax = xTo;
            yMin = yFrom;
            yMax = yTo;
        }
System.out.println("xMin = " + xMin);
System.out.println("xMax = " + xMax);
System.out.println("yMin = " + yMin);
System.out.println("yMax = " + yMax);

        double newXMin = calcuateXNumber(xPixelMin, xMin, xMax, upperLeftX, lowerRightX);
        double newXMax = calcuateXNumber(xPixelMax, xMin, xMax, upperLeftX, lowerRightX);
        double newYMin = calcuateYNumber(yPixelMin, yMin, yMax, upperLeftY, lowerRightY);
        double newYMax = calcuateYNumber(yPixelMax, yMin, yMax, upperLeftY, lowerRightY);

System.out.println("newXMin = " + newXMin);
System.out.println("newXMax = " + newXMax);
System.out.println("newYMin = " + newYMin);
System.out.println("newYMax = " + newYMax);

        synchronized(functions) {
            xFrom = newXMin;
            xTo = newXMax;
            yFrom = newYMax; // hier absichtlich vertauscht!
            yTo = newYMin;
        }

        rangeOfValuesChangeListener.setNewRangeOfValues(newXMin, newXMax,
                newYMax, newYMin); // hier absichtlich vertauscht!

        paintFunctionViewPanelNew();
    }

    private void paintFunctionViewPanelNew() {
        validate();
        invalidate();
        revalidate();
        repaint();
    }

    /** Getter für eine Kopie der Funktionen. */
    public List<Function> getCopyOfFunctions() {
        List<Function> copiedFunctions = new ArrayList<>();
        synchronized(functions) {
            for (Function function : functions) {
                copiedFunctions.add(new Function(function));
            }
        }
        return copiedFunctions;
    }

    /** Entfernt alle Funktionen. */
    public void clearAllFunctions() {
        synchronized(functions) {
            functions.clear();
        }
    }

    /**
     * Fügt eine Funktion hinzu.
     *
     * @param function
     *            Neue Funktion.
     */
    public void addFunction(Function function) {
        synchronized(functions) {
            functions.add(function);
        }
    }

    /**
     * Ändert die Farbe einer Funktion.
     *
     * @param term
     *            Term der Funktion, über diesen wird die passende Funktion ermittelt.
     * @param newColor
     *            Neue Farbe der Funktion.
     */
    public void setFunctionColor(String term, Color newColor) {
        Function function = getFunctionForTerm(term);
        if (null != function) {
            function.setColor(newColor);
        }
    }

    /**
     * Ermittelt eine Funktion.
     *
     * @param term
     *            Term der Funktion, über diesen wird die passende Funktion ermittelt.
     * @return Die zum Term passende Funktion oder null.
     */
    private Function getFunctionForTerm(String term) {
        for (Function function : functions) {
            String functionTerm = function.getTerm();
            if (functionTerm.equals(term)) {
                return function;
            }
        }
        return null;
    }

    /**
     * Entfernt eine Funktion.
     *
     * @param term
     *            Term der Funktion, über diesen wird die passende Funktion ermittelt.
     */
    public void removeFunction(String term) {
        Function function = getFunctionForTerm(term);
        if (null != function) {
            functions.remove(function);
        }
    }

    /*
     * TODO:
     *
     * - Klick mit linker / rechter Maustaste führt zu Vergrößerung /
     *   Verkleinerung (mit Zentrum Klickort?).
     *
     * - Größe und Verhältnis x/y-Auflösung des sichtbaren Bereiches in
     *   Statuszeile anzeigen.
     *
     */

}
