package de.duehl.math.functionviewer.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.filechooser.FileFilter;

import de.duehl.basics.logic.ErrorHandler;
import de.duehl.basics.system.SystemTools;
import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.buttons.painted.ColoredSquareButton;
import de.duehl.swing.ui.buttons.painted.XButton;
import de.duehl.swing.ui.dialogs.base.NonModalFrameDialogBase;
import de.duehl.swing.ui.dialogs.html.ShowHTMLDialog;
import de.duehl.swing.ui.layout.VerticalLayout;
import de.duehl.swing.ui.resources.IconLoader;
import de.duehl.math.functionviewer.logic.CreateHelpPage;
import de.duehl.math.functionviewer.logic.Function;
import de.duehl.math.functionviewer.logic.FunctionViewerLogic;
import de.duehl.math.functionviewer.ui.resources.IconDefinitions;

/**
 * Diese Klasse stellt die grafische Oberfläche des FunctionViewers dar.
 *
 * @version 1.01     2019-07-03
 * @author Christian Dühl
 */

public class FunctionViewerGui extends NonModalFrameDialogBase
    implements RangeOfValuesChangeListener {

    private static final Color NEW_FUNCTION_COLOR = Color.black;

    /** Verbindung zur Logik. */
    private final FunctionViewerLogic logic;

    /** Das Panel, auf dem die Funktionen dargestellt werden. */
    private final FunctionViewPanel functionViewPanel;

    /** Farbe der nächsten neuen Funktion. */
    private Color newFunctionColor; // ist in der Tat veränderlich!

    /** Panel, auf dem die rechts zu sehenden Funktionen dargestellt werden. */
    private final JPanel functionsPanel;

    /**
     * Liste mit den dargestellten Funktionen. Achtung, das ist eine Kopie, direkte Änderungen
     * daran bringen gar nichts, außer zu Anzeigezwecken rechts! Das Original liegt im
     * FunctionViewPanel, Änderungen dorthin werden über den Term übermittelt, anhand dessen die
     * entsprechende Funktion dann ermittelt wird.
     */
    private final List<Function> functions = new ArrayList<>();

    private final JTextField xMinField;

    private final JTextField xMaxField;

    private final JTextField yMinField;

    private final JTextField yMaxField;

    /**
     * Konstruktor.
     *
     * @param logic
     *            Verbindung zur Logik.
     */
    public FunctionViewerGui(FunctionViewerLogic logic) {
        super(loadProgramImage(), "Function Viewer - " + logic.getVersion());
        addClosingWindowListener(logic);

        this.logic = logic;

        GuiTools.setNiceLayoutManager();

        functionViewPanel = new FunctionViewPanel(createErrorHandler(),
                (RangeOfValuesChangeListener) this);
        functionsPanel = new JPanel();

        xMinField = new JTextField();
        xMaxField = new JTextField();
        yMinField = new JTextField();
        yMaxField = new JTextField();

        newFunctionColor = NEW_FUNCTION_COLOR;

        fillDialog();
    }

    private static Image loadProgramImage() {
        IconDefinitions iconDefinitions = new IconDefinitions();
        IconLoader iconLoader = iconDefinitions.createIconLoader();
        return iconLoader.loadProgramIconImage();
    }

    /** Erstellt die Oberfläche. */
    @Override
    protected void populateDialog() {
        add(createButtonPart(), BorderLayout.SOUTH);
        add(createFunctionView(), BorderLayout.CENTER);
        add(createInputPart(), BorderLayout.EAST);

        //setPreferredSize(new Dimension(900, 700));
        setLocation(100, 75);
    }

    /** Erzeugt den Bereich mit den Schaltern unten. */
    private Component createButtonPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(createQuitButton(), BorderLayout.EAST);
        panel.add(createScreenshotButton(), BorderLayout.WEST);

        return panel;
    }

    /** Erzeugt den Button zum Beenden des Programms. */
    private Component createQuitButton() {
        JButton button = new JButton("Beenden");
        //button.setFocusable(false);
        //button.setEnabled(true);
        button.addActionListener(e -> logic.quit());

        return button;
    }

    private Component createScreenshotButton() {
        JButton button = new JButton("Bild speichern");
        button.addActionListener(e -> logic.screenshot());

        return button;
    }

    /** Erstellt den Teil der Gui, auf dem die Funktionen angezeigt werden. */
    private Component createFunctionView() {
        functionViewPanel.setBorder(BorderFactory.createTitledBorder(""));
        functionViewPanel.setPreferredSize(new Dimension(700, 700));
        functionViewPanel.addMouseListener(
                new FunctionViewMouseListener((RangeSelectionReactor) functionViewPanel));
        functionViewPanel.addMouseMotionListener(
                new FunctionViewMouseMotionListener((MouseMotionReactor) functionViewPanel));
        return functionViewPanel;
    }

    /** Erstellt den Teil zur Eingabe neuer Funktionen. */
    private Component createInputPart() {
        JPanel panel = new JPanel();
        panel.setBorder(BorderFactory.createTitledBorder(""));
        panel.setLayout(new BorderLayout());
        panel.setPreferredSize(new Dimension(300, 1));

        panel.add(createMinMaxHelpAndNewFunctionPart(), BorderLayout.NORTH);
        panel.add(createDeleteAllButton(), BorderLayout.SOUTH);
        panel.add(createShownFunctionsPart(), BorderLayout.CENTER);

        return panel;
    }

    private Component createMinMaxHelpAndNewFunctionPart() {
        JPanel panel = new JPanel();

        panel.setLayout(new VerticalLayout(2, VerticalLayout.BOTH));
        panel.add(createMinMaxPanel());
        panel.add(createFunctionHelpButton());
        panel.add(createNewFunctionButton());

        return panel;
    }

    /** Erstellt die Eingabemöglichkeiten für den Wertebereich. */
    private Component createMinMaxPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new VerticalLayout(2, VerticalLayout.BOTH));
        GuiTools.createTitle("Angezeigter Bereich", panel);

        panel.add(createXMinPart());
        panel.add(createXMaxPart());
        panel.add(createYMinPart());
        panel.add(createYMaxPart());

        panel.add(createSetButton());

        return panel;
    }

    private Component createXMinPart() {
        JPanel xMinPanel = new JPanel();
        xMinPanel.setLayout(new BorderLayout());
        JLabel xMinLabel = new JLabel("x min: ");
        xMinPanel.add(xMinLabel, BorderLayout.WEST);
        xMinField.setText(Double.toString(functionViewPanel.getxFrom()));
        xMinPanel.add(xMinField, BorderLayout.CENTER);
        return xMinPanel;
    }

    private Component createXMaxPart() {
        JPanel xMaxPanel = new JPanel();
        xMaxPanel.setLayout(new BorderLayout());
        JLabel xMaxLabel = new JLabel("x max: ");
        xMaxPanel.add(xMaxLabel, BorderLayout.WEST);
        xMaxField.setText(Double.toString(functionViewPanel.getxTo()));
        xMaxPanel.add(xMaxField, BorderLayout.CENTER);
        return xMaxPanel;
    }

    private Component createYMinPart() {
        JPanel yMinPanel = new JPanel();
        yMinPanel.setLayout(new BorderLayout());
        JLabel yMinLabel = new JLabel("y min: ");
        yMinPanel.add(yMinLabel, BorderLayout.WEST);
        yMinField.setText(Double.toString(functionViewPanel.getyFrom()));
        yMinPanel.add(yMinField, BorderLayout.CENTER);
        return yMinPanel;
    }

    private Component createYMaxPart() {
        JPanel yMaxPanel = new JPanel();
        yMaxPanel.setLayout(new BorderLayout());
        JLabel yMaxLabel = new JLabel("y max: ");
        yMaxPanel.add(yMaxLabel, BorderLayout.WEST);
        yMaxField.setText(Double.toString(functionViewPanel.getyTo()));
        yMaxPanel.add(yMaxField, BorderLayout.CENTER);
        return yMaxPanel;
    }

    private Component createSetButton() {
        JButton button = new JButton("Maße setzen");
        button.addActionListener(e -> setRangeOfValues());
        setXAndYFieldsActionListener(e -> button.doClick());
        return button;
    }

    private void setRangeOfValues() {
        String xMinString = xMinField.getText().trim();
        String xMaxString = xMaxField.getText().trim();
        String yMinString = yMinField.getText().trim();
        String yMaxString = yMaxField.getText().trim();

        for (String testString : new String[] { xMinString, xMaxString, yMinString, yMaxString }) {
            if (!testString.matches("^[+-]?\\d+(\\.\\d+)?")) {
                error("Die Eingabe '" + testString + "' ist ungültig.");
                return;
            }
        }

        double xMin = Double.parseDouble(xMinString);
        double xMax = Double.parseDouble(xMaxString);
        double yMin = Double.parseDouble(yMinString);
        double yMax = Double.parseDouble(yMaxString);

        if (xMin >= xMax) {
            error("Die Eingabe xMin='" + xMinString
                    + "', xMax='" + xMaxString
                    + "' ist ungültig.");
            return;
        }
        if (yMin >= yMax) {
            error("Die Eingabe yMin='" + yMinString
                    + "', yMax='" + yMaxString
                    + "' ist ungültig.");
            return;
        }

        setRangeOfValues(xMin, xMax, yMin, yMax);
    }

    /** Setzt den Wertebereich im Funktions-Panel. */
    private void setRangeOfValues(double xMin, double xMax, double yMin, double yMax) {
        functionViewPanel.setxFrom(xMin);
        functionViewPanel.setxTo(xMax);
        functionViewPanel.setyFrom(yMin);
        functionViewPanel.setyTo(yMax);
        functionViewPanel.validate();
        functionViewPanel.invalidate();
        functionViewPanel.revalidate();
        functionViewPanel.repaint();
    }

    private void setXAndYFieldsActionListener(ActionListener reactToEnterListener) {
        xMinField.addActionListener(reactToEnterListener);
        xMaxField.addActionListener(reactToEnterListener);
        yMinField.addActionListener(reactToEnterListener);
        yMaxField.addActionListener(reactToEnterListener);
    }

    /** Erzeugt den Knopf, mit dem alle Funktionen entfernt werden. */
    private Component createDeleteAllButton() {
        JButton button = new JButton("Alles löschen");
        //button.setFocusable(false);
        //button.setEnabled(true);
        button.addActionListener(e -> functionViewPanel.clearAllFunctions());
        return button;
    }

    /** Erzeugt einen Button, der eine Hilfe anzeigt. */
    private Component createFunctionHelpButton() {
        JButton button = new JButton("Hilfe");
        //button.setFocusable(false);
        //button.setEnabled(true);
        button.addActionListener(e -> showHelp());
        return button;
    }

    private void showHelp() {
        String contents = CreateHelpPage.create();
        ShowHTMLDialog dialog = new ShowHTMLDialog("Funktionshilfe", getProgramImage(),
                getLocation());
        dialog.showHtml(contents);
        dialog.setVisible(true);
    }

    /** Erzeugt den Bereich zum Erzeugen einer neuen Funktion. */
    private Component createNewFunctionButton() {
        JPanel panel = new JPanel();
        panel.setBorder(BorderFactory.createTitledBorder("Neue Funktion"));
        panel.setLayout(new VerticalLayout(2, VerticalLayout.BOTH));

        panel.add(createNewFunctionTitleTabel());

        JTextField termField = createTermField(); // brauchen wir unten !
        panel.add(termField);

        panel.add(createButtonPanel(termField));

        return panel;
    }

    private Component createNewFunctionTitleTabel() {
        return new JLabel("Term von f(x):");
    }

    private JTextField createTermField() {
        JTextField termField = new JTextField();
        SwingUtilities.invokeLater(() -> termField.requestFocus());
        return termField;
    }

    private JPanel createButtonPanel(JTextField termField) {
        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(1, 2));

        JButton colorButton = createColorButton();
        panel.add(colorButton);

        JButton addFunctionButton = createAddFunctionButton(termField, colorButton);
        panel.add(addFunctionButton);

        termField.addActionListener(e -> addFunctionButton.doClick());

        return panel;
    }

    private JButton createColorButton() {
        JButton colorButton = new JButton("Farbe");
        colorButton.addActionListener(e -> colorButtonClicked(colorButton));
        return colorButton;
    }

    private JButton createAddFunctionButton(JTextField termField, JButton colorButton) {
        JButton addFunctionButton = new JButton("hinzufügen");
        addFunctionButton.addActionListener(e -> addFunctionButtonClicked(termField, colorButton));
        return addFunctionButton;
    }

    private void colorButtonClicked(JButton colorButton) {
        Color selectedColor = JColorChooser.showDialog(
                getFrame(),
                "Eine Farbe für die Funktion wählen:",
                newFunctionColor
                );
        if (null != selectedColor) {
            newFunctionColor = selectedColor;
            colorButton.setForeground(newFunctionColor);
        }
    }

    private void addFunctionButtonClicked(JTextField termField, JButton colorButton) {
        String term = termField.getText().trim();

        if (existsTermInFunctions(term)) {
            warning("Es gibt bereits eine Funktion mit dem Term '" + term + "'!");
            termField.requestFocus();
            return;
        }

        createFunction(term, termField, colorButton);
    }

    private void createFunction(String term, JTextField termField, JButton colorButton) {
        try {
            tryToCreateFunction(term, termField, colorButton);
        }
        catch (Exception exception) {
            error("Es trat ein Fehler bei der Berechnung der Funktion auf.", exception);
        }
    }

    private void tryToCreateFunction(String term, JTextField termField, JButton colorButton) {
        Function function = new Function(term, newFunctionColor);
        functionViewPanel.addFunction(function);
        functions.add(function);
        actualizeShownFunctionsPart();
        functionViewPanel.repaint();

        /* Initialisierungen für die nächste neue Funktion: */
        termField.setText("");
        newFunctionColor = NEW_FUNCTION_COLOR; // oder nicht?
        colorButton.setForeground(newFunctionColor);
        termField.requestFocus();
    }

    /**
     * Prüft, ob es eine Funktion mit einem bestimmten Term gibt.
     *
     * @param term
     *            Term der Funktion.
     * @return true genau dann, wenn es eine Funktion mit diesem Term gibt.
     */
    private boolean existsTermInFunctions(String term) {
        String trimmedTerm = term.trim().replaceAll("\\s+", "");

        for (Function function : functions) {
            String functionTerm = function.getTerm();
            functionTerm = functionTerm.trim().replaceAll("\\s+", " ");
            if (functionTerm.equals(trimmedTerm)) {
                return true;
            }
        }

        return false;
    }

    /** Erzeugt den Bereich, der die verwendeten Funktionen anzeigt. */
    private Component createShownFunctionsPart() {
        functionsPanel.setBorder(BorderFactory.createTitledBorder("Funktionen"));
        functionsPanel.setLayout(new VerticalLayout(2, VerticalLayout.BOTH));

        return new JScrollPane(functionsPanel);
    }

    private void actualizeShownFunctionsPart() {
        functionsPanel.removeAll();
        Dimension buttonSize = new Dimension(25, 25);

        for (Function function : functions) {
            addFunction(buttonSize, function);
        }

        //functionsPanel.validate();
        //functionsPanel.invalidate();
        functionsPanel.revalidate();
        functionsPanel.repaint();
        revalidate();
        //repaint();
    }

    private void addFunction(Dimension buttonSize, Function function) {
        JPanel panel = new JPanel();
        functionsPanel.add(panel);
        panel.setLayout(new BorderLayout());

        panel.add(createFunctionTermLabel(function), BorderLayout.CENTER);
        panel.add(
                createFunctionButtonPanel(buttonSize, function, createFunctionTermLabel(function)),
                BorderLayout.EAST);
    }

    private JLabel createFunctionTermLabel(Function function) {
        JLabel termLabel = new JLabel(function.getTerm());
        termLabel.setForeground(function.getColor());
        return termLabel;
    }

    private JPanel createFunctionButtonPanel(Dimension buttonSize, Function function,
            JLabel termLabel) {
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout(1, 2));

        buttonPanel.add(createFunctionColorButton(buttonSize, function, termLabel));
        buttonPanel.add(createRemoveFunctionButton(buttonSize, function));
        return buttonPanel;
    }

    private ColoredSquareButton createFunctionColorButton(Dimension buttonSize, Function function,
            JLabel termLabel) {
        ColoredSquareButton button = new ColoredSquareButton(function.getColor());
        button.setForeground(function.getColor());
        button.setPreferredSize(buttonSize);
        button.addActionListener(e -> functionColorButtonPressed(function, termLabel, button));
        return button;
    }

    private void functionColorButtonPressed(Function function, JLabel termLabel,
            ColoredSquareButton colorButton) {
        Color selectedColor = JColorChooser.showDialog(
                getFrame(),
                "Eine Farbe für die Funktion wählen:",
                newFunctionColor
                );
        if (null != selectedColor) {
            colorButton.setForeground(selectedColor);
            termLabel.setForeground(selectedColor);
            function.setColor(selectedColor);
            functionViewPanel.setFunctionColor(function.getTerm(),
                    selectedColor);
            colorButton.setSquareColor(selectedColor);
            functionViewPanel.repaint();
        }
    }

    private JButton createRemoveFunctionButton(Dimension buttonSize, Function function) {
        JButton button = new XButton();
        button.setPreferredSize(buttonSize);
        button.addActionListener(e -> removeFunction(function));
        return button;
    }

    private void removeFunction(Function function) {
        functions.remove(function);
        functionViewPanel.removeFunction(function.getTerm());
        actualizeShownFunctionsPart();
        functionViewPanel.repaint();
    }

    /** Beendet die Gui. */
    public void quit() {
        closeDialog();
    }

    /** Zeigt dem Benutzer eine Warnung an. */
    private void warning(String message) {
        ErrorHandler error = createErrorHandler();
        error.warning(message);
    }

    /** Zeigt dem Benutzer einen Fehler an. */
    private void error(String message) {
        ErrorHandler error = createErrorHandler();
        error.error(message);
    }

    /** Zeigt dem Benutzer einen Fehler mit Exception an. */
    public void error(String message, Exception exception) {
        ErrorHandler error = createErrorHandler();
        error.error(message, exception);
    }

    /**
     * Setzt den neuen Wertebereich.
     *
     * @param newXMin
     *            linker x-Wert.
     * @param newXMax
     *            rechter x-Wert.
     * @param newYMin
     *            oberer y-Wert.
     * @param newYMax
     *            unterer y-Wert.
     */
    @Override
    public void setNewRangeOfValues(double newXMin, double newXMax, double newYMin,
            double newYMax) {
        String xMinString = numberToString(newXMin);
        String xMaxString = numberToString(newXMax);
        String yMinString = numberToString(newYMin);
        String yMaxString = numberToString(newYMax);
        xMinField.setText(xMinString);
        xMaxField.setText(xMaxString);
        yMinField.setText(yMinString);
        yMaxField.setText(yMaxString);
    }

    /**
     * Erstellt aus einer Zahl als double einen String zur Anzeige in der Oberfläche.
     *
     * @param number
     *            Zu wandelnde Zahl.
     * @return Zahl als string.
     */
    private String numberToString(double number) {
        String numberString = Double.toString(number);
        numberString = numberString.replaceAll("0+$", "");
        numberString = numberString.replaceAll("\\.$", "");
        return numberString;
    }

    /** Fragte den Benutzer nach dem Namen für den Screenshot der Funktionen. */
    public String askUserAboutNameForScreenshot() {
        FileFilter fileFilter = GuiTools.createExtensionFileFilter(".png");
        return GuiTools.saveFileAs(getFrame(), SystemTools.getHomeDirectory(), fileFilter);
    }

    /** Fertigt einen den Screenshot der Funktionen an. */
    public RenderedImage getSnapshot() {
        JPanel view = functionViewPanel;
        int width = view.getWidth();
        int height = view.getHeight();

        BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);

        synchronized (view.getTreeLock()) {
            view.paint(image.createGraphics());
        }

        return image;
    }

}
