package de.duehl.basics.system;

/*
 * Copyright 2019 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.io.FileDescriptor;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.io.UnsupportedEncodingException;
import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 * Eine Klasse mit System-Tools.
 *
 * @version 1.01     2019-06-27
 * @author Christian Dühl
 */

public class SystemTools {

    /**
     * Findet heraus, welches Betriebssystem verwendet wird.
     *
     * @return Betriebssytemart
     */
    public static OperationSystem determineOperationSystem() {
        String os = System.getProperty("os.name", "unknown system");
        //System.out.println("Operating system: " + os);

        OperationSystem ourSystem = OperationSystem.UNKNOWN;
        if ("Mac OS X".equals(os)) {
            ourSystem = OperationSystem.OSX;
        }
        else if (os.startsWith("Windows")) {   // "Windows XP"
            ourSystem = OperationSystem.WINDOWS;
        }

        return ourSystem;
    }

    /**
     * Lässt den aktuellen Thread die angegebene Zeit in Millisekunden schlafen.
     *
     * @param milliseconds
     *            Zeit in Millisekunden.
     */
    public static void sleep(long milliseconds) {
        try {
            Thread.sleep(milliseconds);
        }
        catch (InterruptedException e) {
            // do nothing
        }
    }

    /**
     * Lässt den aktuellen Thread die angegebene Zeit in Sekunden schlafen.
     *
     * @param seconds
     *            Zeit in Sekunden.
     */
    public static void sleepSeconds(long seconds) {
        sleep(1000L * seconds);
    }

    /**
     * Lässt den aktuellen Thread die angegebene Zeit in Minuten schlafen.
     *
     * @param minutes
     *            Zeit in Minuten.
     */
    public static void sleepMinutes(long minutes) {
        sleepSeconds(60L * minutes);
    }

    /** Setzt für die Ausgabe auf STDOUT die richtige Codepage für cmd.exe. */
    public static void setOutToCp850() {
        try {
            System.setOut(new PrintStream(new FileOutputStream(FileDescriptor.out), true, "CP850"));
        }
        catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
    }

//    /** Setzt für die Ausgabe auf STDOUT die richtige Codepage für cmd.exe. */
//    public static void setInToCp850() {
//        /*
//        FileInputStream stream = new FileInputStream(FileDescriptor.in);
//        InputStreamReader reader = new InputStreamReader(stream, "CP850");
//        System.setIn(reader);
//        */
//        try {
//            FileInputStream fdIn = new FileInputStream(FileDescriptor.in);
//
//            System.setIn(new BufferedInputStream(fdIn));
//        }
//        catch (UnsupportedEncodingException e) {
//            e.printStackTrace();
//        }
//    }

    /** Gibt den Namen des Benutzer (in kleine Buchstaben gewandelt) zurück. */
    public static String getUserName() {
        /*
         * Da Windows die Groß- / Kleinschreibung offenbar egal ist und mal usr514, mal USR514
         * zurückkommt, müssen wir es selbst normieren:
         */
        String name = System.getProperty("user.name", "").toLowerCase();
        return name;
    }

    /** Gibt den Namen des Rechners zurück. */
    public static String getComputerName() {
        try {
            InetAddress localMachine = java.net.InetAddress.getLocalHost();
            return localMachine.getHostName();
        }
        catch (UnknownHostException e) {
            throw new RuntimeException(e);
        }
    }

    /** Ermittelt das Home-Directory (~ unter Linux, c:\Users\UserName unter Windows). */
    public static String getHomeDirectory() {
        String home = System.getProperty("user.home", ".");
        return home;
    }

    /** Gibt die IP4-Adresse des Rechners in der Form 192.168.30.119 zurück. */
    public static String getLocalhost() {
        try {
            return tryGetLocalhost();
        }
        catch (Exception exception) {
            throw new RuntimeException(exception);
        }
    }

    private static String tryGetLocalhost() throws UnknownHostException {
        InetAddress inetAddress = InetAddress.getLocalHost();
        String address = inetAddress.getHostAddress();
        return address;
    }

}
