package de.duehl.basics.io;

/*
 * Copyright 2019 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import static org.junit.Assert.*;

import java.io.File;
import java.net.URL;
import java.util.List;

import org.junit.Test;

import de.duehl.basics.io.data.FileWithModificationTime;
import de.duehl.basics.io.exceptions.IORuntimeException;

public class FileHelperTest {

    private final String path;
    private final String multipeSubsDir;

    private String path() {
        String className = getClass().getName();
        int dotIndex = className.indexOf(".");
        while (dotIndex > -1) {
            className = className.substring(dotIndex + 1); // endet nicht auf .
            dotIndex = className.indexOf(".");
        }

        URL url = getClass().getResource(className + ".class");
        File f = new File(url.getPath());
        f = new File(f.getParent());
        f = new File(f.getParent());
        f = new File(f.getParent());
        f = new File(f.getParent());
        f = new File(f.getParent());
        f = new File(f.getParent());
        String p = f.getPath() + File.separator + "data" + File.separator
                + "testfiles" + File.separator;
        return p;
    }

    public FileHelperTest() {
        path = path();
        multipeSubsDir = FileHelper.concatPathes(path, "mehrere_unterverzeichnisse");
    }


    /* -------------------------- TESTS -------------------------- */


    @Test
    public void lastSlash1() {
        String input = "/";
        int expected = 0;
        int actual = FileHelper.lastSlash(input);
        assertEquals(expected, actual);
    }

    @Test
    public void lastSlash2() {
        String input = "\\";
        int expected = 0;
        int actual = FileHelper.lastSlash(input);
        assertEquals(expected, actual);
    }

    @Test
    public void lastSlash3() {
        String input = "123";
        int expected = -1;
        int actual = FileHelper.lastSlash(input);
        assertEquals(expected, actual);
    }

    @Test
    public void lastSlash4() {
        String input = "/123/567";
        int expected = 4;
        int actual = FileHelper.lastSlash(input);
        assertEquals(expected, actual);
    }

    @Test
    public void lastSlash5() {
        String input = "\\123\\567";
        int expected = 4;
        int actual = FileHelper.lastSlash(input);
        assertEquals(expected, actual);
    }

    @Test
    public void lastSlash6() {
        String input = "\\///\\///";
        int expected = 7;
        int actual = FileHelper.lastSlash(input);
        assertEquals(expected, actual);
    }

    @Test
    public void insertBeforeExtensionSimple() {
        String org = "foo.bar";
        String ins = FileHelper.insertBeforeExtension(org, "_baz");
        assertEquals("foo_baz.bar", ins);
    }

    @Test
    public void insertBeforeExtensionMoreDots() {
        String org = "foo.bar.baz";
        String ins = FileHelper.insertBeforeExtension(org, "_BAZ");
        assertEquals("foo.bar_BAZ.baz", ins);
    }

    @Test
    public void insertBeforeExtensionWithPath() {
        String org = "/users/justme/foo.bar.baz";
        String ins = FileHelper.insertBeforeExtension(org, "_BAZ");
        assertEquals("/users/justme/foo.bar_BAZ.baz", ins);
    }

    @Test
    public void insertBeforeExtensionWithPathAndEvenMoreDots() {
        String org = "/users.silly/./../just.me/foo.bar.baz.fooooobar";
        String ins = FileHelper.insertBeforeExtension(org, "_BAZ");
        assertEquals("/users.silly/./../just.me/foo.bar.baz_BAZ.fooooobar", ins);
    }

    @Test
    public void insertBeforeExtensionWithoutAnyDot() {
        String org = "foo_bar";
        String ins = FileHelper.insertBeforeExtension(org, "_baz");
        assertEquals("foo_bar_baz", ins);
    }

    @Test
    public void insertBeforeExtensionWithoutAnyDotButDotInPath() {
        String org = "/users.silly/./../just.me/foo_bar";
        String ins = FileHelper.insertBeforeExtension(org, "_baz");
        assertEquals("/users.silly/./../just.me/foo_bar_baz", ins);
    }

    @Test
    public void insertBeforeExtensionDotAtEnd() {
        String org = "foo.";
        String ins = FileHelper.insertBeforeExtension(org, "_baz");
        assertEquals("foo_baz.", ins);
    }

    @Test
    public void insertBeforeExtensionDotAtStart() {
        String org = ".foo";
        String ins = FileHelper.insertBeforeExtension(org, "_baz");
        assertEquals("_baz.foo", ins);
    }

    @Test
    public void insertBeforeExtensionDotAtStartWithPath() {
        String org = "/users.silly/./../just.me/.foo";
        String ins = FileHelper.insertBeforeExtension(org, "_baz");
        assertEquals("/users.silly/./../just.me/_baz.foo", ins);
    }

    @Test
    public void removeExtensionOnBareName() {
        String input = "bla.txt";
        String actual = FileHelper.removeExtension(input);
        String expected = "bla";
        assertEquals(expected, actual);
    }

    @Test
    public void removeExtensionOnFullName() {
        String input = "c:/foo/bar.txt";
        String actual = FileHelper.removeExtension(input);
        String expected = "c:/foo/bar";
        assertEquals(expected, actual);
    }

    @Test
    public void removeExtensionOnOnlyExtension() {
        String input = ".foo";
        String actual = FileHelper.removeExtension(input);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void removeExtensionOnOnlyExtensionWithPath() {
        String input = "/users.silly/./../just.me/.foo";
        String actual = FileHelper.removeExtension(input);
        String expected = "/users.silly/./../just.me/";
        assertEquals(expected, actual);
    }

    @Test
    public void removeExtensionWithoutExtension() {
        String input = "foo";
        String actual = FileHelper.removeExtension(input);
        String expected = "foo";
        assertEquals(expected, actual);
    }

    @Test
    public void determineExtensionOnBareName() {
        String input = "bla.txt";
        String actual = FileHelper.determineExtension(input);
        String expected = ".txt";
        assertEquals(expected, actual);
    }

    @Test
    public void determineExtensionOnFullName() {
        String input = "c:/foo/bar.html";
        String actual = FileHelper.determineExtension(input);
        String expected = ".html";
        assertEquals(expected, actual);
    }

    @Test
    public void determineExtensionOnOnlyExtension() {
        String input = ".foo";
        String actual = FileHelper.determineExtension(input);
        String expected = ".foo";
        assertEquals(expected, actual);
    }

    @Test
    public void determineExtensionOnOnlyExtensionWithPath() {
        String input = "/users.silly/./../just.me/.foo";
        String actual = FileHelper.determineExtension(input);
        String expected = ".foo";
        assertEquals(expected, actual);
    }

    @Test
    public void determineExtensionWithoutExtension() {
        String input = "foo";
        String actual = FileHelper.determineExtension(input);
        String expected = FileHelper.NO_EXTENSION_FOUND;
        assertEquals(expected, actual);
    }

    @Test
    public void determineExtensionWithoutExtensionWithPath() {
        String input = "c:/foo/foo";
        String actual = FileHelper.determineExtension(input);
        String expected = FileHelper.NO_EXTENSION_FOUND;
        assertEquals(expected, actual);
    }

    @Test
    public void concatPathesNoSlash() {
        String part1 = "a/b/c";
        String part2 = "d/e/f";
        //String expected = "a/b/c/d/e/f"; // unix
        String expected = "a/b/c\\d/e/f"; // windows
        String actual = FileHelper.concatPathes(part1, part2);
        assertEquals(expected, actual);
    }

    @Test
    public void concatPathesSlashBehindFirst() {
        String part1 = "a/b/c/";
        String part2 = "d/e/f";
        String expected = "a/b/c/d/e/f";
        String actual = FileHelper.concatPathes(part1, part2);
        assertEquals(expected, actual);
    }

    @Test
    public void concatPathesSlashBeforeSecond() {
        String part1 = "a/b/c";
        String part2 = "/d/e/f";
        String expected = "a/b/c/d/e/f";
        String actual = FileHelper.concatPathes(part1, part2);
        assertEquals(expected, actual);
    }

    @Test
    public void concatPathesBothWithSlashes() {
        String part1 = "a/b/c/";
        String part2 = "/d/e/f";
        String expected = "a/b/c/d/e/f";
        String actual = FileHelper.concatPathes(part1, part2);
        assertEquals(expected, actual);
    }

    @Test
    public void concatPathesBothWithBackslashes() {
        String part1 = "a\\b\\c\\";
        String part2 = "\\d\\e\\f";
        String expected = "a\\b\\c\\d\\e\\f";
        String actual = FileHelper.concatPathes(part1, part2);
        assertEquals(expected, actual);
    }

    @Test
    public void concatPathesMultipleEmpty() {
        String expected = "";
        String actual = FileHelper.concatPathesMultiple();
        assertEquals(expected, actual);
    }

    @Test
    public void concatPathesMultipleOnePart() {
        String expected = "foo";
        String actual = FileHelper.concatPathesMultiple("foo");
        assertEquals(expected, actual);
    }

    @Test
    public void concatPathesMultipleTwoParts() {
        String expected = "foo\\bar";
        String actual = FileHelper.concatPathesMultiple("foo", "bar");
        assertEquals(expected, actual);
    }

    @Test
    public void concatPathesMultipleTwoPartsAndAnEmptyPartBetweenThem() {
        String expected = "foo\\bar";
        String actual = FileHelper.concatPathesMultiple("foo", "", "bar");
        assertEquals(expected, actual);
    }

    @Test
    public void concatPathesMultipleThreeParts() {
        String expected = "foo\\bar\\baz";
        String actual = FileHelper.concatPathesMultiple("foo", "bar", "baz");
        assertEquals(expected, actual);
    }

    @Test
    public void splitCsvSemikolonLine1() {
        String in = "\"DokNr\";\"Vortext\"";
        List<String> parts = FileHelper.splitCsvSemikolonLine(in);
        assertEquals(2, parts.size());
        assertEquals("DokNr", parts.get(0));
        assertEquals("Vortext", parts.get(1));
    }

    @Test
    public void normalizeToSlahes1() {
        String expected = "foo/bar/baz";
        String actual = FileHelper.normalizeToSlahes("foo\\bar\\baz");
        assertEquals(expected, actual);
    }

    @Test
    public void normalizeToSlahes2() {
        String expected = "foo/bar/baz";
        String actual = FileHelper.normalizeToSlahes("foo/bar/baz");
        assertEquals(expected, actual);
    }

    @Test
    public void normalizeToBackSlahes1() {
        String expected = "foo\\bar\\baz";
        String actual = FileHelper.normalizeToBackSlahes("foo\\bar\\baz");
        assertEquals(expected, actual);
    }

    @Test
    public void normalizeToBackSlahes2() {
        String expected = "foo\\bar\\baz";
        String actual = FileHelper.normalizeToBackSlahes("foo/bar/baz");
        assertEquals(expected, actual);
    }

    @Test
    public void getDirName() {
        String filename = "C:\\test\\README.txt";
        String actual = FileHelper.getDirName(filename);
        String expected = "C:\\test\\";
        assertEquals(expected, actual);
    }

    @Test
    public void getDirNameEndingWithDirAndSlash() {
        String filename = "C:\\test\\foo\\";
        String actual = FileHelper.getDirName(filename);
        String expected = "C:\\test\\";
        assertEquals(expected, actual);
    }

    @Test
    public void getDirNameEndingWithDirWithoutSlash() {
        String filename = "C:\\test\\foo";
        String actual = FileHelper.getDirName(filename);
        String expected = "C:\\test\\";
        assertEquals(expected, actual);
    }

    @Test
    public void getDirNameWithoutLastSlash() {
        String filename = "C:\\test\\README.txt";
        String actual = FileHelper.getDirNameWithoutLastSlash(filename);
        String expected = "C:\\test";
        assertEquals(expected, actual);
    }

    @Test
    public void getDirNameWithoutLastSlashEndingWithDirAndSlash() {
        String filename = "C:\\test\\foo\\";
        String actual = FileHelper.getDirNameWithoutLastSlash(filename);
        String expected = "C:\\test";
        assertEquals(expected, actual);
    }

    @Test
    public void getDirNameWithoutLastSlashEndingWithDirWithoutSlash() {
        String filename = "C:\\test\\foo";
        String actual = FileHelper.getDirNameWithoutLastSlash(filename);
        String expected = "C:\\test";
        assertEquals(expected, actual);
    }

    @Test
    public void isDirectoryExisting() {
        String dir = "c:/temp";
        assertTrue(FileHelper.isDirectory(dir));
    }

    @Test
    public void isDirectoryNotExisting() {
        String dir = "! ;;; ::: not a existing, valid directory path ::: ;;; !";
        assertFalse(FileHelper.isDirectory(dir));
    }

    @Test
    public void isNotADirectoryExisting() {
        String dir = "c:/temp";
        assertFalse(FileHelper.isNotADirectory(dir));
    }

    @Test
    public void isNotADirectoryNotExisting() {
        String dir = "! ;;; ::: not a existing, valid directory path ::: ;;; !";
        assertTrue(FileHelper.isNotADirectory(dir));
    }

    @Test
    public void normalisePath() {
        String filename = "c:/tmp/../tmp/./foo.txt";
        String actual = FileHelper.normalisePath(filename);
        String expected = "C:\\tmp\\foo.txt";
        assertEquals(expected, actual);
    }

    @Test
    public void getSystemDisplayName() {
        String path = "C:\\";
        String actual = FileHelper.getSystemDisplayName(path);
        String expected = "OS";
        assertEquals(expected, actual);
    }

    @Test
    public void getSystemDisplayNameWithLowerDriveLetter() {
        String path = "c:\\";
        String actual = FileHelper.getSystemDisplayName(path);
        String expected = "OS";
        assertEquals(expected, actual);
    }

    @Test (expected = RuntimeException.class)
    public void getSystemDisplayNameFailOnWrongSlash() {
        String path = "C:/";
        FileHelper.getSystemDisplayName(path);
    }

    @Test (expected = RuntimeException.class)
    public void getSystemDisplayNameFailOnWrongPath() {
        String path = "\\\\Cluster1\\Projekte1\\";
        FileHelper.getSystemDisplayName(path);
    }

    @Test
    public void getSystemDisplayNameWithDeeperPath() {
        String path = "C:\\temp\\";
        String actual = FileHelper.getSystemDisplayName(path);
        String expected = "OS";
        assertEquals(expected, actual);
    }

    @Test
    public void getBareNameWithBackslashBehind() {
        String filename = "C:\\temp\\";
        String actual = FileHelper.getBareName(filename);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void getBareNameWithSlashBehind() {
        String filename = "C:/temp/";
        String actual = FileHelper.getBareName(filename);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void getBareName01() {
        String filename = "C:/temp/foo";
        String actual = FileHelper.getBareName(filename);
        String expected = "foo";
        assertEquals(expected, actual);
    }

    @Test
    public void getBareName02() {
        String filename = "C:/temp/foo/bar";
        String actual = FileHelper.getBareName(filename);
        String expected = "bar";
        assertEquals(expected, actual);
    }

    @Test
    public void getBareName03() {
        String filename = "C:/temp/foo/bar\\baz";
        String actual = FileHelper.getBareName(filename);
        String expected = "baz";
        assertEquals(expected, actual);
    }

    @Test
    public void getBareNameFromEmptyString() {
        String filename = "";
        String actual = FileHelper.getBareName(filename);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void buildFilenameFromPath01() {
        String path = "C:/temp/foo/bar\\baz";
        String actual = FileHelper.buildFilenameFromPath(path);
        String expected = "C_temp_foo_bar_baz";
        assertEquals(expected, actual);
    }

    @Test
    public void buildFilenameFromPath02() {
        String path = "\\\\Cluster1\\Projekte1\\Bla";
        String actual = FileHelper.buildFilenameFromPath(path);
        String expected = "Cluster1_Projekte1_Bla";
        assertEquals(expected, actual);
    }

    @Test
    public void fileSizeExisting() {
        String filename = path + "testfile1.txt";
        long actual = FileHelper.fileSize(filename);
        long expected = 24;
        assertEquals(expected, actual);
    }

    @Test
    public void fileSizeExistingWithFile() {
        File file = new File(path + "testfile1.txt");
        long actual = FileHelper.fileSize(file);
        long expected = 24;
        assertEquals(expected, actual);
    }

    @Test (expected = IORuntimeException.class)
    public void fileSizeFailureNotExisting() {
        String filename = "\\\\Cluster1\\Projekte1\\Bla";
        FileHelper.fileSize(filename);
    }

    @Test
    public void filesHaveEqualSizeTrue() {
        String filenameA = path + "testfile1.txt";
        String filenameB = path + "testfile1.txt";
        assertTrue(FileHelper.filesHaveEqualSize(filenameA, filenameB));
    }

    @Test
    public void filesHaveEqualSizeFalse() {
        String filenameA = path + "testfile1.txt";
        String filenameB = path + "file_writer_demo3.txt";
        assertFalse(FileHelper.filesHaveEqualSize(filenameA, filenameB));
    }

    @Test
    public void filesHaveEqualSizeWithFileTrue() {
        File fileA = new File(path + "testfile1.txt");
        File fileB = new File(path + "testfile1.txt");
        assertTrue(FileHelper.filesHaveEqualSize(fileA, fileB));
    }

    @Test
    public void filesHaveEqualSizeWithFileFalse() {
        File fileA = new File(path + "testfile1.txt");
        File fileB = new File(path + "file_writer_demo3.txt");
        assertFalse(FileHelper.filesHaveEqualSize(fileA, fileB));
    }

    @Test
    public void filesHaveSameLastModifiedTimeTrue() {
        String filenameA = path + "testfile1.txt";
        String filenameB = path + "testfile1.txt";
        assertTrue(FileHelper.filesHaveSameLastModifiedTime(filenameA, filenameB));
    }

    @Test
    public void filesHaveSameLastModifiedTimeFalse() {
        String filenameA = path + "testfile1.txt";
        String filenameB = path + "file_writer_demo3.txt";
        assertFalse(FileHelper.filesHaveSameLastModifiedTime(filenameA, filenameB));
    }

    @Test
    public void filesHaveSameLastModifiedTimeWithFileTrue() {
        File fileA = new File(path + "testfile1.txt");
        File fileB = new File(path + "testfile1.txt");
        assertTrue(FileHelper.filesHaveSameLastModifiedTime(fileA, fileB));
    }

    @Test
    public void filesHaveSameLastModifiedTimeWithFileFalse() {
        File fileA = new File(path + "testfile1.txt");
        File fileB = new File(path + "file_writer_demo3.txt");
        assertFalse(FileHelper.filesHaveSameLastModifiedTime(fileA, fileB));
    }

    @Test
    public void filesAreEqualEnoughTrue() {
        String filenameA = path + "testfile1.txt";
        String filenameB = path + "testfile1.txt";
        assertTrue(FileHelper.filesAreEqualEnough(filenameA, filenameB));
    }

    @Test
    public void filesAreEqualEnoughFalse() {
        String filenameA = path + "testfile1.txt";
        String filenameB = path + "file_writer_demo3.txt";
        assertFalse(FileHelper.filesAreEqualEnough(filenameA, filenameB));
    }

    @Test
    public void filesAreEqualEnoughWithFileTrue() {
        File fileA = new File(path + "testfile1.txt");
        File fileB = new File(path + "testfile1.txt");
        assertTrue(FileHelper.filesAreEqualEnough(fileA, fileB));
    }

    @Test
    public void filesAreEqualEnoughWithFileFalse() {
        File fileA = new File(path + "testfile1.txt");
        File fileB = new File(path + "file_writer_demo3.txt");
        assertFalse(FileHelper.filesAreEqualEnough(fileA, fileB));
    }

    @Test
    public void exchangeExtensionWithPathWithoutDotBeforeNewExtension() {
        String filename = "c:/temp/foo.txt";
        String actual = FileHelper.exchangeExtension(filename, "html");
        String expected = "c:/temp/foo.html";
        assertEquals(expected, actual);
    }

    @Test
    public void exchangeExtensionWithPathWithDotBeforeNewExtension() {
        String filename = "c:/temp/foo.txt";
        String actual = FileHelper.exchangeExtension(filename, ".html");
        String expected = "c:/temp/foo.html";
        assertEquals(expected, actual);
    }

    @Test
    public void exchangeExtensionWithPathWithoutDotBeforeNewExtensionMultipleDot() {
        String filename = "c:/temp/foo.bar.txt";
        String actual = FileHelper.exchangeExtension(filename, "html");
        String expected = "c:/temp/foo.bar.html";
        assertEquals(expected, actual);
    }

    @Test
    public void exchangeExtensionWithPathWithDotBeforeNewExtensionMultipleDot() {
        String filename = "c:/temp/foo.bar.txt";
        String actual = FileHelper.exchangeExtension(filename, ".html");
        String expected = "c:/temp/foo.bar.html";
        assertEquals(expected, actual);
    }

    @Test
    public void exchangeExtensionWithoutPathWithoutDotBeforeNewExtension() {
        String filename = "foo.txt";
        String actual = FileHelper.exchangeExtension(filename, "html");
        String expected = "foo.html";
        assertEquals(expected, actual);
    }

    @Test
    public void exchangeExtensionWithoutPathWithDotBeforeNewExtension() {
        String filename = "foo.txt";
        String actual = FileHelper.exchangeExtension(filename, ".html");
        String expected = "foo.html";
        assertEquals(expected, actual);
    }

    @Test
    public void exchangeExtensionWithoutPathWithoutDotBeforeNewExtensionMultipleDot() {
        String filename = "foo.bar.txt";
        String actual = FileHelper.exchangeExtension(filename, "html");
        String expected = "foo.bar.html";
        assertEquals(expected, actual);
    }

    @Test
    public void exchangeExtensionWithoutPathWithDotBeforeNewExtensionMultipleDot() {
        String filename = "foo.bar.txt";
        String actual = FileHelper.exchangeExtension(filename, ".html");
        String expected = "foo.bar.html";
        assertEquals(expected, actual);
    }

    @Test
    public void exchangeExtensionWithoutPathWithoutDotBeforeNewExtensionMultipleDotAndAtStart() {
        String filename = ".foo.bar.txt";
        String actual = FileHelper.exchangeExtension(filename, "html");
        String expected = ".foo.bar.html";
        assertEquals(expected, actual);
    }

    @Test
    public void exchangeExtensionWithoutPathWithDotBeforeNewExtensionMultipleDotAndAtStart() {
        String filename = ".foo.bar.txt";
        String actual = FileHelper.exchangeExtension(filename, ".html");
        String expected = ".foo.bar.html";
        assertEquals(expected, actual);
    }

    @Test (expected = IllegalArgumentException.class)
    public void exchangeExtensionFailesNoDot() {
        String filename = "hier ist kein Punkt!";
        FileHelper.exchangeExtension(filename, ".html");
    }

    @Test (expected = IllegalArgumentException.class)
    public void exchangeExtensionFailesOnlyDotAtStart() {
        String filename = ".foo";
        FileHelper.exchangeExtension(filename, ".html");
    }

    @Test (expected = IllegalArgumentException.class)
    public void exchangeExtensionFailesDotAtEnd() {
        String filename = "c:/temp/foo.bar.txt.";
        FileHelper.exchangeExtension(filename, ".html");
    }

    @Test (expected = IllegalArgumentException.class)
    public void exchangeExtensionFailesComplexExtension() {
        String filename = "c:/temp/foo.bar.übelübel";
        FileHelper.exchangeExtension(filename, ".html");
    }

    @Test (expected = IllegalArgumentException.class)
    public void exchangeExtensionFailesComplexExtension2() {
        String filename = "c:/temp/foo.bar,txt";
        FileHelper.exchangeExtension(filename, ".html");
    }

    @Test (expected = IllegalArgumentException.class)
    public void exchangeExtensionFailesComplexExtension3() {
        String filename = "c:/temp/foo.txt_so_nicht";
        FileHelper.exchangeExtension(filename, ".html");
    }

    @Test (expected = IllegalArgumentException.class)
    public void doNotExchangeAgainstSameExtension() {
        String filename = "c:/temp/foo.html";
        FileHelper.exchangeExtension(filename, ".html");
    }

    @Test
    public void replaceUncPathesR() {
        String filename = "\\\\Cluster1\\Projekte1\\HRTag\\HR_Strukt_HuP";
        String actual = FileHelper.replaceUncPathes(filename);
        String expected = "r:\\HRTag\\HR_Strukt_HuP";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceUncPathesI1() {
        String filename = "\\\\Cluster2\\Projekte2\\Schufa\\Impressum";
        String actual = FileHelper.replaceUncPathes(filename);
        String expected = "i:";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceUncPathesI2() {
        String filename = "\\\\Cluster2\\Projekte2\\Schufa\\Impressum\\data";
        String actual = FileHelper.replaceUncPathes(filename);
        String expected = "i:\\data";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceUncPathesS() {
        String filename = "\\\\Cluster2\\Projekte2\\Schufa\\Mtest";
        String actual = FileHelper.replaceUncPathes(filename);
        String expected = "s:\\Schufa\\Mtest";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceUncPathesUBig() {
        String filename = "\\\\Cluster1\\User1\\CD\\ImpressumProjekt";
        String actual = FileHelper.replaceUncPathes(filename);
        String expected = "u:\\ImpressumProjekt";
        assertEquals(expected, actual);
    }

    @Test
    public void replaceUncPathesUSmall() {
        String filename = "\\\\Cluster1\\user1\\CD\\ImpressumProjekt";
        String actual = FileHelper.replaceUncPathes(filename);
        String expected = "u:\\ImpressumProjekt";
        assertEquals(expected, actual);
    }

    @Test
    public void findFilesInMainDirectoryFindOneDotHtmlFile() {
        String start = path + "findFilesInMainDirectoryNio2";
        String extension = ".html";
        List<String> files = FileHelper.findFilesInMainDirectory(start, extension);
        assertEquals(1, files.size());
        assertEquals(start + "\\test1.html", files.get(0));
    }

    @Test
    public void findFilesInMainDirectoryFindOneHtmlFile() {
        String start = path + "findFilesInMainDirectoryNio2";
        String extension = "html";
        List<String> files = FileHelper.findFilesInMainDirectory(start, extension);
        assertEquals(1, files.size());
        assertEquals(start + "\\test1.html", files.get(0));
    }

    @Test
    public void findFilesInMainDirectoryFindTwoDotTextFiles() {
        String start = path + "findFilesInMainDirectoryNio2";
        String extension = ".txt";
        List<String> files = FileHelper.findFilesInMainDirectory(start, extension);
        assertEquals(2, files.size());
        assertEquals(start + "\\test2.txt", files.get(0));
        assertEquals(start + "\\test3.txt", files.get(1));
    }

    @Test
    public void findFilesInMainDirectoryFindTwoTextFilese() {
        String start = path + "findFilesInMainDirectoryNio2";
        String extension = "txt";
        List<String> files = FileHelper.findFilesInMainDirectory(start, extension);
        assertEquals(2, files.size());
        assertEquals(start + "\\test2.txt", files.get(0));
        assertEquals(start + "\\test3.txt", files.get(1));
    }

    @Test
    public void findFilesInMainDirectoryFindOneDotHtmlFileNio2() {
        String start = path + "findFilesInMainDirectoryNio2";
        String extension = ".html";
        List<String> files = FileHelper.findFilesInMainDirectoryNio2(start, extension);
        assertEquals(1, files.size());
        assertEquals(start + "\\test1.html", files.get(0));
    }

    @Test
    public void findFilesInMainDirectoryFindOneHtmlFileNio2() {
        String start = path + "findFilesInMainDirectoryNio2";
        String extension = "html";
        List<String> files = FileHelper.findFilesInMainDirectoryNio2(start, extension);
        assertEquals(1, files.size());
        assertEquals(start + "\\test1.html", files.get(0));
    }

    @Test
    public void findFilesInMainDirectoryFindTwoDotTextFilesNio2() {
        String start = path + "findFilesInMainDirectoryNio2";
        String extension = ".txt";
        List<String> files = FileHelper.findFilesInMainDirectoryNio2(start, extension);
        assertEquals(2, files.size());
        assertEquals(start + "\\test2.txt", files.get(0));
        assertEquals(start + "\\test3.txt", files.get(1));
    }

    @Test
    public void findFilesInMainDirectoryFindTwoTextFileseNio2() {
        String start = path + "findFilesInMainDirectoryNio2";
        String extension = "txt";
        List<String> files = FileHelper.findFilesInMainDirectoryNio2(start, extension);
        assertEquals(2, files.size());
        assertEquals(start + "\\test2.txt", files.get(0));
        assertEquals(start + "\\test3.txt", files.get(1));
    }

    @Test
    public void containsFiles() {
        String start = path + "findFilesInMainDirectoryNio2";
        String extension = "txt";
        boolean found = FileHelper.containsFiles(start, extension);
        assertTrue(found);
    }

    @Test
    public void containsFilesWithDot() {
        String start = path + "findFilesInMainDirectoryNio2";
        String extension = ".txt";
        boolean found = FileHelper.containsFiles(start, extension);
        assertTrue(found);
    }

    @Test
    public void findAllFilesInMainDirectory() {
        String start = path + "findFilesInMainDirectoryNio2";
        List<String> files = FileHelper.findAllFilesInMainDirectory(start);
        assertEquals(3, files.size());
        assertEquals(start + "\\test1.html", files.get(0));
        assertEquals(start + "\\test2.txt", files.get(1));
        assertEquals(start + "\\test3.txt", files.get(2));
    }

    @Test
    public void findAllFilesInMainDirectoryNio2() {
        String start = path + "findFilesInMainDirectoryNio2";
        List<String> files = FileHelper.findAllFilesInMainDirectoryNio2(start);
        assertEquals(3, files.size());
        assertEquals(start + "\\test1.html", files.get(0));
        assertEquals(start + "\\test2.txt", files.get(1));
        assertEquals(start + "\\test3.txt", files.get(2));
    }

    @Test
    public void findFiles() {
        String start = path + "findFilesInMainDirectoryNio2";
        List<String> files = FileHelper.findFiles(start, ".htm", ".html", ".txt");
        assertEquals(5, files.size());
        assertEquals(start + "\\subdir\\test1.html", files.get(0));
        assertEquals(start + "\\subdir\\test2.txt", files.get(1));
        assertEquals(start + "\\test1.html", files.get(2));
        assertEquals(start + "\\test2.txt", files.get(3));
        assertEquals(start + "\\test3.txt", files.get(4));
    }

    @Test
    public void findFilesNio2() {
        String start = path + "findFilesInMainDirectoryNio2";
        List<String> files = FileHelper.findFilesNio2(start, ".htm", ".html", ".txt");
        //for (String file : files)  System.out.println(file);
        assertEquals(5, files.size());
        assertEquals(start + "\\subdir\\test1.html", files.get(0));
        assertEquals(start + "\\subdir\\test2.txt", files.get(1));
        assertEquals(start + "\\test1.html", files.get(2));
        assertEquals(start + "\\test2.txt", files.get(3));
        assertEquals(start + "\\test3.txt", files.get(4));
    }

    @Test
    public void findFilesNio2UpperCaseExtension() {
        String start = path + "findFilesInMainDirectoryNio2";
        List<String> files = FileHelper.findFilesNio2(start, "HTML", "TXT");
        assertEquals(5, files.size());
        assertEquals(start + "\\subdir\\test1.html", files.get(0));
        assertEquals(start + "\\subdir\\test2.txt", files.get(1));
        assertEquals(start + "\\test1.html", files.get(2));
        assertEquals(start + "\\test2.txt", files.get(3));
        assertEquals(start + "\\test3.txt", files.get(4));
    }

    @Test
    public void findFilesNio2Old() {
        String start = path + "findFilesInMainDirectoryNio2";
        List<String> files = FileHelper.findFilesNio2Old(start, ".htm", ".html", ".txt");
        assertEquals(5, files.size());
        assertEquals(start + "\\subdir\\test1.html", files.get(0));
        assertEquals(start + "\\subdir\\test2.txt", files.get(1));
        assertEquals(start + "\\test1.html", files.get(2));
        assertEquals(start + "\\test2.txt", files.get(3));
        assertEquals(start + "\\test3.txt", files.get(4));
    }

    @Test
    public void findFilesNio2OldUpperCaseExtension() {
        String start = path + "findFilesInMainDirectoryNio2";
        List<String> files = FileHelper.findFilesNio2Old(start, "HTML", "TXT");
        assertEquals(5, files.size());
        assertEquals(start + "\\subdir\\test1.html", files.get(0));
        assertEquals(start + "\\subdir\\test2.txt", files.get(1));
        assertEquals(start + "\\test1.html", files.get(2));
        assertEquals(start + "\\test2.txt", files.get(3));
        assertEquals(start + "\\test3.txt", files.get(4));
    }

    @Test
    public void findAllFilesNio2() {
        String start = path + "findFilesInMainDirectoryNio2";
        List<String> files = FileHelper.findAllFilesNio2(start);
        assertEquals(5, files.size());
        assertEquals(start + "\\subdir\\test1.html", files.get(0));
        assertEquals(start + "\\subdir\\test2.txt", files.get(1));
        assertEquals(start + "\\test1.html", files.get(2));
        assertEquals(start + "\\test2.txt", files.get(3));
        assertEquals(start + "\\test3.txt", files.get(4));
    }

    @Test
    public void createDirectoryWithMissingSubdirectoriesIfNotExists() {
        prepareForCreatingSubdirectories();

        String dir = FileHelper.concatPathesMultiple(multipeSubsDir, "sub1", "sub2", "sub3");
        //System.out.println(dir);
        assertFalse(FileHelper.isDirectory(dir));
        FileHelper.createDirectoryWithMissingSubdirectoriesIfNotExists(dir);
        assertTrue(FileHelper.isDirectory(dir));

        removeForCreatingSubdirectories();
    }

    @Test
    public void createMissingDirectoriesForFile() {
        prepareForCreatingSubdirectories();


        String filename = FileHelper.concatPathesMultiple(multipeSubsDir, "sub1", "sub2", "sub3",
                "hallo.txt");
        //System.out.println(filename);
        assertFalse(FileHelper.isFile(filename));
        FileHelper.createMissingDirectoriesForFile(filename);
        FileHelper.writeTextToFile("Hallo!", filename);
        assertTrue(FileHelper.isFile(filename));

        removeForCreatingSubdirectories();
    }

    private void prepareForCreatingSubdirectories() {
        assertTrue(FileHelper.isDirectory(path));
        assertFalse(FileHelper.isDirectory(multipeSubsDir));
    }

    private void removeForCreatingSubdirectories() {
        FileHelper.deleteTree(multipeSubsDir);

        assertTrue(FileHelper.isDirectory(path));
        assertFalse(FileHelper.isDirectory(multipeSubsDir));
    }

    @Test
    public void determineAllFilesInMainDirectoryDescendingByTime() {
        // path = C:\Users\Anwender\Daten\java\workspace\de.duehl.basics\data\testfiles\
        String dir = FileHelper.concatPathes(path, "fixed_times");

        List<FileWithModificationTime> files =
                FileHelper.determineAllFilesInMainDirectoryDescendingByTime(dir);

        assertEquals(9, files.size());

        assertEquals(dir + "\\example.html", files.get(0).getFilename());
        assertEquals(dir + "\\file_writer_demo.txt", files.get(1).getFilename());
        assertEquals(dir + "\\file_writer_demo2.txt", files.get(2).getFilename());
        assertEquals(dir + "\\file_writer_demo3.txt", files.get(3).getFilename());
        assertEquals(dir + "\\file_writer_demo_write_fields.txt", files.get(4).getFilename());
        assertEquals(dir + "\\file_writer_demo_write_fields_list.txt", files.get(5).getFilename());
        assertEquals(dir + "\\file_writer_demo_write_fields_vararg.txt", files.get(6).getFilename());
        assertEquals(dir + "\\example.txt", files.get(7).getFilename());
        assertEquals(dir + "\\testfile1.txt", files.get(8).getFilename());

        assertEquals("20190124_121035", files.get(0).getModificationTime());
        assertEquals("20190118_094601", files.get(1).getModificationTime());
        assertEquals("20190118_094601", files.get(2).getModificationTime());
        assertEquals("20190118_094601", files.get(3).getModificationTime());
        assertEquals("20190118_094601", files.get(4).getModificationTime());
        assertEquals("20190118_094601", files.get(5).getModificationTime());
        assertEquals("20190118_094601", files.get(6).getModificationTime());
        assertEquals("20180824_131948", files.get(7).getModificationTime());
        assertEquals("20140330_124526", files.get(8).getModificationTime());
    }

    @Test
    public void prepareToFilenamePartEmptyString() {
        String complexSearch = "";
        String actual = FileHelper.prepareToFilenamePart(complexSearch);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void prepareToFilenamePartThreeUndescores() {
        String complexSearch = "___";
        String actual = FileHelper.prepareToFilenamePart(complexSearch);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void prepareToFilenamePartRealInput() {
        String complexSearch = "name:Heins AND strasse:Decius*";
        String actual = FileHelper.prepareToFilenamePart(complexSearch);
        String expected = "name_Heins_AND_strasse_Decius";
        assertEquals(expected, actual);
    }

    @Test
    public void prepareToFilenamePartRealInputWithSpaces() {
        String complexSearch = "          name:Heins    AND   strasse:Decius*    ";
        String actual = FileHelper.prepareToFilenamePart(complexSearch);
        String expected = "name_Heins_AND_strasse_Decius";
        assertEquals(expected, actual);
    }

}
