package de.duehl.swing.ui.dialogs.html;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JRootPane;

import de.duehl.swing.ui.key.BindKeysOnRootPane;

/**
 * Diese Klasse ist die Abstrakte Basis für die Anzeige von Html in einem JFrame und einem JDialog.
 *
 * @version 1.01     2018-04-12
 * @author Christian Dühl
 */

abstract class ShowHtml {

    /** Panel in dem das HTML dargestellt wird. */
    private final HtmlPanel htmlPanel;

    /** Gesamter Panel. */
    private final JPanel panel;

    /** Panel in den durch den Aufrufer einen weiteren Button hinzugefügt werden kann. */
    private final JPanel extraButtonsPanel;

    /**
     * Konstruktor.
     *
     * @param title
     *            Titel des Dialogs.
     * @param programImage
     *            Image vom aufrufenden Programm.
     * @param parentLocation
     *            Lokation des Aufrufenden Dialoges.
     */
    public ShowHtml() {
        htmlPanel = new HtmlPanel();
        extraButtonsPanel = createExtraButtonPanel();
        panel = buildThePanel();
    }

    private JPanel createExtraButtonPanel() {
        JPanel panel = new JPanel();
        //panel.setLayout(new FlowLayout(FlowLayout.LEFT)); // das macht die andren Buttons größer
        panel.setLayout(new BorderLayout());

        return panel;
    }

    /** Erzeugt den Panel. */
    private JPanel buildThePanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(htmlPanel.getHtmlPanel(), BorderLayout.CENTER);
        panel.add(createButtonPanel(), BorderLayout.SOUTH);

        return panel;
    }

    private Component createButtonPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(createQuitButton(), BorderLayout.EAST);
        panel.add(createToggleAndExtraButtonPart(), BorderLayout.WEST);

        return panel;
    }

    private Component createQuitButton() {
        JButton button = new JButton();
        button.setText("Beenden");
        button.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent arg0) {
                closeDialog();
            }
        });
        return button;
    }

    private Component createToggleAndExtraButtonPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(createToggleButton(), BorderLayout.WEST);
        panel.add(extraButtonsPanel, BorderLayout.EAST);

        return panel;
    }

    private Component createToggleButton() {
        JButton button = new JButton();
        button.setText("HTML / Sourcecode");
        button.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent arg0) {
                htmlPanel.switchCard();
            }
        });
        return button;
    }

    /** Fügt einen weiteren Button zum Dialog hinzu. */
    public void addExtraButton(JButton button) {
        //extraButtonsPanel.add(button);
        extraButtonsPanel.add(button, BorderLayout.CENTER);
    }

    /** Getter für den gesamten Panel. */
    public JPanel getPanel() {
        return panel;
    }

    /** Schließt den Dialog. */
    abstract protected void closeDialog();

    /** Erstellt Tastenkombinationen. */
    protected void setKeyBindings(JRootPane rootPane) {
        BindKeysOnRootPane bind = new BindKeysOnRootPane(rootPane);
        bind.setKeyBindingEscape(() -> closeDialog());
        bind.setKeyBindingHome(() -> htmlPanel.setScrollPaneToMinimum());
        bind.setKeyBindingEnd(() -> htmlPanel.setScrollPaneToMaximum());
        bind.setKeyBindingCtrlHome(() -> htmlPanel.setScrollPaneToMinimum());
        bind.setKeyBindingCtrlEnd(() -> htmlPanel.setScrollPaneToMaximum());
        bind.setKeyBindingPageUp(() -> htmlPanel.setScrollPaneToPreviousSection());
        bind.setKeyBindingPageDown(() -> htmlPanel.setScrollPaneToNextSection());
    }

    /**
     * Schreibt den Text in den Hauptteil des Dialogs.
     *
     * @param url
     *            Datei, aus der der Inhalt gelesen wird.
     */
    public void showHtml(URL url) {
        htmlPanel.showHtml(url);
    }

    /**
     * Schreibt den Text in den Hauptteil des Dialogs.
     *
     * @param htmlContents
     *            Darzustellender Text.
     */
    public void showHtml(String htmlContents) {
        htmlPanel.showHtml(htmlContents);
    }

}
